#include "Game.h"
#include <sstream>
#include <math.h>
#include "Lemming.h"

Game::Game(int ScreenWidth, int ScreenHeight, int new_framesPerSecond,
		int newZoom, bool Sound_enabled, int Buffer) {
	framesPerSecond = new_framesPerSecond - 7;
	zoom = newZoom;
	sound_enabled = Sound_enabled;
	screen_size = new SDL_Rect();
	screen_size->x = screen_size->y = 0;
	screen_size->w = ScreenWidth * zoom;
	screen_size->h = 160 * zoom;

	screenWidth = ScreenWidth;
	screenHeight = ScreenHeight;

	myGameChange.index = INVALID_INDEX;
	myGameChange.new_state = INVALID_NEW_STATE;
	musicwason = false;

	fontRenderer = FontRenderer::Instance();
	utility = GraphicsUtility::Instance();
	samples = new SamplePlayer(sound_enabled);
	statusPanel = new StatusPanel(ScreenWidth, 160/*ScreenHeight*/, 0, (LEVEL_Y_SIZE - Y_OFFSET) * 2, samples,
			zoom);
	currentLevel = new Level(ScreenWidth, ScreenHeight);
	sprites = new SpriteStore();

	gameType = StatusPanel::SinglePlayer;
	//Create all the Lemmings:
	for (int n = 0; n < max_num_lemmings; n++) {
		ulemmings.lemmings[n] = new Lemming(6 * zoom, 9 * zoom, 0, 0, zoom);
		lemmings_copy[n] = new Lemming(6 * zoom, 9 * zoom, 0, 0, zoom);
	}

	debugMode = false;
	effects_only = false;
	music_now_on = false;
	ResetMultiplayer();
	dataBufferer = new DataBuffer();
	playbackMode = false;

	showLemmingInfo = true;
	SetInspectLemmingMode(false);
	super_lem_hack = false;
}

Game::~Game() {
	delete (fontRenderer);
	delete (samples);
	delete (screen_size);

	//delete all the Lemmings:
	for (int n = 0; n < max_num_lemmings; n++) {
		delete (ulemmings.lemmings[n]);
		delete (lemmings_copy[n]);
	}

	// for some reason these cause seg faults
	delete (statusPanel);
	delete (currentLevel);
}

void Game::SetInspectLemmingMode(bool on) {
	inspectLemmingMode = on;
	statusPanel->HideSelection(on);
}

bool Game::GetInspectLemmingMode() {
	return inspectLemmingMode;
}

void Game::Pause() {
	pause = !pause;
}

void Game::ResetGameToStart() {
	selectedIndex = INVALID_INDEX;
	selectedSprite = NULL;
	gameData.blueOut = gameData.greenOut = gameData.releasePoint
			= gameData.numberReleased = gameData.totalOut = gameData.numHome
					= gameData.thisTotalOut = 0;
	gameData.ticksSinceNukeClick = gameData.ticksSinceGameStart = 0;
	blueNuked = greenNuked = gameData.totalledUp = false;

	gameData.minReleaseRate = currentLevel->GetReleaseRate();
	timeRemaining = currentLevel->GetTimeInMinutes() * 60;
	//the following 2 lines ensure we get a lemming straight away
	gameData.releaseCounter = (60 - ((int) currentLevel->GetReleaseRate() * 55
			/ 100));

	statusPanel->SetGameStats(0, 0, 0, 0, 0, 0);
	frameIndex = 1;

	//kill all the Lemmings wuhuhaahaahaa!
	for (int n = 0; n < max_num_lemmings; n++) {
		ulemmings.lemmings[n]->SetDead();
	}

	statusPanel->ClearTotals();
}

int Game::LemmingsNeededPercent() {
	return (currentLevel->GetLemmingsToBeSaved() * 100)
			/ (currentLevel->GetNumLemmings());
}

void Game::Init(StatusPanel::GameType Sender) {
	ResetGameToStart();

	suppress_tool_decrease = false;
	pause = false;
	nukeAll = false;

	music_now_on = false;
	gameType = Sender;
	selectedSprite = NULL;

	fpsCount = framesPerSecond;
	if(super_lem_hack){
		printf("super lemming!\n");
		fps = framesPerSecond * 4;
	}
	else{
		fps = framesPerSecond;
	}

	statusPanel->viewport.x = (currentLevel->GetCamStartX() - screenWidth / 2
			+ 16) * zoom;

	printf("statusPanel->viewport.x: %d\n", statusPanel->viewport.x);

	if (gameType != StatusPanel::SinglePlayer) {
		statusPanel->viewport.x -= screenWidth / 4;
	}

	statusPanel->SetGameMode(gameType);
	storeChanges = true;

	gameCountDown = -1;
}

bool Game::ConnectToGame(StatusPanel::GameType Sender, std::string IPAddress) {
	bool connected = false;
	printf("Connecting machines\n");

	if (gameType == StatusPanel::Client) {
		connected = dataBufferer->ConnectToServer(IPAddress);
	} else if (gameType == StatusPanel::Server) {
		connected = dataBufferer->ConnectToClient();
	}

	// this is the fancy (no-working) method
	//if (connected)
	//{
	//	dataBufferer->StartThread();
	//}

	return connected;
}

SDL_Surface* Game::GetLevelPreview() {
	return currentLevel->GetLevelPreview();
}

bool Game::WaitForClientToConnect() {
	return dataBufferer->IsClientConnected();
}

bool Game::SetClientLevel(int newLevel) {
	return dataBufferer->SetClientLevel(newLevel);
}

int Game::GetLevelFromServer() {
	return dataBufferer->GetLevelFromServer();
}

void Game::NormalSpeedPlaybackGame() {

}

unsigned int nCurrentMaxFrameIndex;
void Game::StartPlayback() {
	playbackMode = true;
	nCurrentMaxFrameIndex = frameIndex;
	ResetGameToStart();
	currentLevel->ReRenderBackground();

}

/*
 void Game::PlaybackGame() {
 DataBuffer::NetData change1;
 DataBuffer::NetData change2;
 unsigned int currentMaxFrameIndex = frameIndex;
 ResetGameToStart();
 currentLevel->ReRenderBackground();

 storeChanges = false;
 suppress_tool_decrease = true;
 samples->DisableSamples();

 //there can only be a maximum of 2 changes per frame (as it's only two player)
 //now go through every store of every change, re-doing them
 for (unsigned int tFrame = 1; tFrame < currentMaxFrameIndex + 1; tFrame++) {
 //gp through every state change of the game so far:
 change1.frameIndex = change2.frameIndex = 0;
 change1.new_state = change2.new_state = INVALID_NEW_STATE;
 change1.index = change2.index = INVALID_INDEX;

 for (unsigned int nf = 0; nf < dataBufferer->size(); nf++) {
 DataBuffer::NetData* netData = dataBufferer->at(nf);
 if (netData->frameIndex == tFrame) {
 if (change1.frameIndex == 0) {
 change1.frameIndex = netData->frameIndex;
 change1.new_state = netData->new_state;
 change1.index = netData->index;
 } else if (change2.frameIndex == 0) {
 change2.frameIndex = netData->frameIndex;
 change2.new_state = netData->new_state;
 change2.index = netData->index;
 } else
 printf("something's gone horribly wrong\n");
 }
 }

 ProcessFrame(&change1, &change2);
 }

 samples->EnableSamples();
 suppress_tool_decrease = false;
 storeChanges = true;
 //printf("finished back game\n");
 }
 */

/*
 void Game::StoreGameState() {
 gameData_copy.ticksSinceGameStart = gameData.ticksSinceGameStart;
 gameData_copy.ticksSinceNukeClick = gameData.ticksSinceNukeClick;
 gameData_copy.totalledUp = gameData.totalledUp;
 gameData_copy.blueOut = gameData.blueOut;
 gameData_copy.greenOut = gameData.greenOut;
 gameData_copy.releasePoint = gameData.releasePoint;
 gameData_copy.minReleaseRate = gameData.minReleaseRate;
 gameData_copy.nukeCount = gameData.nukeCount;
 gameData_copy.releaseCounter = gameData.releaseCounter;
 gameData_copy.numberReleased = gameData.numberReleased;
 gameData_copy.totalOut = gameData.totalOut;
 gameData_copy.thisTotalOut = gameData.thisTotalOut;
 gameData_copy.numHome = gameData.numHome;
 for (int n = 0; n < max_num_lemmings; n++)
 lemmings_copy[n]->MakeEqualTo(ulemmings.lemmings[n]);
 }*/

void Game::LoadLevel(LevelLoader* levelLoader) {
	//printf("(Game) In game LoadLevel\n");
	//printf("(Game) loading level: %s\n", levelLoader->GetLevel()->c_str());
	if (currentLevel != NULL) {
		printf("(Game) currentLevel is Not Null at least\n");
		currentLevel->LoadLevel(levelLoader->GetLevel());
		printf("(Game) Creating overview image next:\n");
		statusPanel->CreateOverviewImage(currentLevel->GetBackground());
		levelTune = Mix_LoadMUS(levelLoader->GetSong().c_str());
	}
}

void Game::Load2PlayerLevel(LevelLoader* levelLoader) {
	currentLevel->LoadLevel(levelLoader->GetTwoPlayerLevel(0));
	twoplayerhack = levelLoader->Get2PlayerLevelHack();
	if (twoplayerhack->flags_swapped) {
		currentLevel->SwapFlags();
	}
	statusPanel->CreateOverviewImage(currentLevel->GetBackground());
	if (sound_enabled)
		levelTune = Mix_LoadMUS(levelLoader->GetSong().c_str());
}

int Game::getDistanceToLemming(Lemming* l, int xp, int yp) {
	int t, v;
	if (xp > l->position->x) {
		t = xp - l->position->x;
		if (yp > l->position->y) {
			v = yp - l->position->y;
		} else {
			v = l->position->y - yp;
		}
	} else {
		t = l->position->x - xp;
		if (yp > l->position->y) {
			v = yp - l->position->y;
		} else {
			v = l->position->y - yp;
		}
	}
	return sqrt((t * t) + (v * v));
}

void Game::SetMouseClick(int x, int y, long leftButtonDown) {
	mouseX = x;
	mouseY = y;
	int iconSelected = -1;

	int xx = mouseX;
	int yy = mouseY;
	int selectedCount = 0;

	//correct for (this is shit) multiplayer positioning
	int potdist = 999;

	bool selectThis = false;
	int iconClicked = 0;
	bool InspectModeDoAction = false;
	if (leftButtonDown) {
		if (!inspectLemmingMode) {
			selectedSprite = NULL;
			selectedIndex = INVALID_INDEX;
		}

		iconClicked = statusPanel->SelectIcon(x, y);
	} else if (!inspectLemmingMode) {
		selectedSprite = NULL;
		selectedIndex = INVALID_INDEX;
	}

	//Get the selected Icon
	iconSelected = statusPanel->GetSelectedIcon();

	if (inspectLemmingMode && iconClicked != 0) {
		//do the action on the currently selected lemming
		if (selectedSprite != NULL) {
			//yep, do it then!
			InspectModeDoAction = true;
		}
	} else if(!inspectLemmingMode || (inspectLemmingMode && iconClicked == 0 && leftButtonDown) ) {
		Lemming* tmpNewSelection = selectedSprite;
		//is the mouse over a lemmings?
		for (int n = 0; n < max_num_lemmings; n++) {
			Lemming* potentialNewSelected = ulemmings.lemmings[n];

			//first should we even do this one?
			selectThis = false;

			switch (gameType) {
			case (StatusPanel::Server):
				if (potentialNewSelected->GetPlayerNum() == 1)
					selectThis = true;
				break;
			case (StatusPanel::Client):
				if (potentialNewSelected->GetPlayerNum() == 2)
					selectThis = true;
				break;
			case (StatusPanel::SinglePlayer):
				selectThis = true;
				break;
			}

			if (potentialNewSelected != NULL && selectThis
					&& potentialNewSelected->GetAliveState()
					&& !(potentialNewSelected->GetState() & EXPLODING)
					&& !(potentialNewSelected->GetState() & ARRIVED_HOME)) {
				if (potentialNewSelected->IsPointWithin(xx
						+ statusPanel->viewport.x, yy + statusPanel->viewport.y)) {
					//this lemming is a candidate
					if (tmpNewSelection == NULL) {
						//yes, there's nowt else so a've it
						potdist = getDistanceToLemming(potentialNewSelected, xx
								+ statusPanel->viewport.x, yy
								+ statusPanel->viewport.y);
						tmpNewSelection = ulemmings.lemmings[n];
						selectedIndex = n;
						selectedCount++;
					} else {
						//this is a bit more difficult then
						//we need to decide if this one is a better
						//candidate or not.

						//the first choice should be which is closer
						int dist = getDistanceToLemming(potentialNewSelected,
								xx + statusPanel->viewport.x, yy
										+ statusPanel->viewport.y);
						if (dist < potdist) {
							//CLOSER
							//if potential guy is already whatever we're trying to set then don't bother EVER
							if (!(potentialNewSelected->GetState()
									& iconSelected)) {
								//he's not one, so use him..perhaps
								if (!((selectedSprite->GetState() & BLOCKER)
										&& iconSelected == NUKE)) {
									tmpNewSelection = ulemmings.lemmings[n];
									selectedIndex = n;
									selectedCount++;
								} else if (((potentialNewSelected->GetState()
										& BLOCKER) && iconSelected == NUKE)) {
									//if both the same then the closer one wins out!
									tmpNewSelection = ulemmings.lemmings[n];
									selectedIndex = n;
									selectedCount++;
								} else if (potentialNewSelected->GetState()
										== selectedSprite->GetState()) {
									//if both the same then the closer one wins out!
									tmpNewSelection = ulemmings.lemmings[n];
									selectedIndex = n;
									selectedCount++;
								} else if (selectedSprite->GetState()
										& iconSelected) {
									//if the one we already have is already this
									tmpNewSelection = ulemmings.lemmings[n];
									selectedIndex = n;
									selectedCount++;
								}
							}
						} else {
							//NOT CLOSER but still inside, so now further checks:
							//printf("new lem is NOT closer\n");
							//Is the potential one a blocker and are we tyring to nuke? (and it's not already counting down)
							if ((potentialNewSelected->GetState() & BLOCKER)
									&& iconSelected == NUKE
									&& (potentialNewSelected->CountingDown()
											!= -2)) {
								//yes, so let user nuke blocker
								tmpNewSelection = ulemmings.lemmings[n];
								selectedIndex = n;
								selectedCount++;
							} else if (!(tmpNewSelection->GetState() & BLOCKER)
									&& iconSelected == NUKE
									&& (selectedSprite->CountingDown() != -2)) {
								tmpNewSelection = ulemmings.lemmings[n];
								selectedIndex = n;
								selectedCount++;
							} else if ((tmpNewSelection->GetState() & BLOCKER)
									&& (iconSelected == BUILDER || iconSelected
											== MINER || iconSelected == BASHER)) {
								selectedSprite = ulemmings.lemmings[n];
								selectedIndex = n;
								selectedCount++;
							} else if (tmpNewSelection->GetState()
									& iconSelected) {
								//if the one we already have is already this
								tmpNewSelection = ulemmings.lemmings[n];
								selectedIndex = n;
								selectedCount++;
							}
						}
					}
				}
			}

			if ((tmpNewSelection != NULL && leftButtonDown)
					|| !inspectLemmingMode) {
				selectedSprite = tmpNewSelection;
			}
		}
	}

	if (gameData.ticksSinceNukeClick > 0) {
		gameData.ticksSinceNukeClick++;
	}

	int doaction = INVALID_NEW_STATE;
	if ((leftButtonDown && !inspectLemmingMode) || InspectModeDoAction) {
		switch (iconSelected) {
		case (DIGGER):
			if (selectedSprite != NULL) {
				if (currentLevel->ToolAvailable(Level::DIGGER_TOOL)) {
					if (!currentLevel->IsPointMetalArea(
							selectedSprite->position->x,
							selectedSprite->position->y + 2)) {
						doaction = 7;
					} else
						samples->PlaySample(CHINK);
				}
			}
			break;
		case (MINER):
			if (selectedSprite != NULL) {
				if (currentLevel->ToolAvailable(Level::MINER_TOOL)) {
					if (!currentLevel->IsPointMetalArea(
							selectedSprite->position->x
									+ (selectedSprite->GetState() & GOING_LEFT) ? -6
									: 8, selectedSprite->position->y + 2)) {
						doaction = 6;
					} else
						samples->PlaySample(CHINK);
				}
			}
			break;
		case (BASHER):
			if (selectedSprite != NULL) {
				if (currentLevel->ToolAvailable(Level::BASHER_TOOL)) {
					if (!currentLevel->IsPointExpandedMetalArea(
							selectedSprite->position->x
									+ (selectedSprite->GetState() & GOING_LEFT) ? -8
									: 10, selectedSprite->position->y - 2)) {
						doaction = 5;
					} else
						samples->PlaySample(CHINK);
				}
			}
			break;
		case (BUILDER):
			if (selectedSprite != NULL && currentLevel->ToolAvailable(
					Level::BUILDER_TOOL))
				doaction = 4;
			break;
		case (BLOCKER):
			if (selectedSprite != NULL && currentLevel->ToolAvailable(
					Level::BLOCKER_TOOL))
				doaction = 3;
			break;
		case (NUKE):
			if (selectedSprite != NULL && currentLevel->ToolAvailable(
					Level::NUKE_TOOL))
				doaction = 2;
			break;
		case (FLOATER):
			if (selectedSprite != NULL && currentLevel->ToolAvailable(
					Level::FLOATER_TOOL))
				doaction = 1;
			break;
		case (CLIMBER):
			if (selectedSprite != NULL && currentLevel->ToolAvailable(
					Level::CLIMBER_TOOL))
				doaction = 0;
			break;
		default:
			doaction = INVALID_NEW_STATE;
			selectedIndex = INVALID_INDEX;
			break;
		}
	} else if (selectedSprite != NULL) {
		//inspect lemming mode, we just make him the latest selected one...and he's done
		selectedCount = 1;
	}

	/***/

	if (iconClicked == NUKE_BUTTON) {
		printf("nuke clicked\n");
		if (gameData.ticksSinceNukeClick > 0 && gameData.ticksSinceNukeClick
				< 8) {
			if (gameType == StatusPanel::SinglePlayer) {
				if (!nukeAll) {
					// only nuke all when game has officially started
					if (gameData.ticksSinceGameStart >= GAME_START_PAUSE) {
						//always play oh no sample if single player
						samples->PlaySample(OHNO);
						nukeAll = true;
						selectedIndex = INVALID_INDEX;
						gameData.nukeCount = 0;
					}
				}
			} else {
				// only nuke all when game has officially started
				if (gameData.ticksSinceGameStart >= GAME_START_PAUSE) {
					if (gameType == StatusPanel::Client) {
						//	if(!greenNuked){
						printf("Client did a nuke!\n");
						doaction = 8;
						selectedIndex = INVALID_INDEX;
					} else //if (!blueNuked) {
					{
						printf("Server did a nuke!\n");
						doaction = 8;
						selectedIndex = INVALID_INDEX;
					}
				}
			}
			gameData.ticksSinceNukeClick = 0;
		} else
			gameData.ticksSinceNukeClick = 1;
	} else if (iconClicked == PAUSE_BUTTON) {
		if (gameType == StatusPanel::SinglePlayer) {
			pause = !pause;
		}
	}

	//now, if we need to do something
	if (doaction != INVALID_NEW_STATE) {
		//myGameChange.frameIndex = frameIndex;
		myGameChange.index = (byte) selectedIndex;
		myGameChange.new_state = (byte) doaction;
		printf("myGameChange: lemming index: %d, state: %d\n",
				myGameChange.index, myGameChange.new_state);
	}

	//Shows the type of Lemming in the status panel
	if (selectedSprite != NULL && showLemmingInfo) {
		if ((selectedSprite->GetState() & FALLING))
			statusPanel->SetSelectedStatus("Faller", selectedCount);
		else if ((selectedSprite->GetState() & CLIMBER)
				&& (selectedSprite->GetState() & FLOATER))
			statusPanel->SetSelectedStatus("Athlete", selectedCount);
		else if ((selectedSprite->GetState() & FLOATER))
			statusPanel->SetSelectedStatus("Floater", selectedCount);
		else if ((selectedSprite->GetState() & CLIMBER))
			statusPanel->SetSelectedStatus("Climber", selectedCount);
		else if (selectedSprite->GetState() & WALKING)
			statusPanel->SetSelectedStatus("Walker", selectedCount);
		else if (selectedSprite->GetState() & NUKE)
			statusPanel->SetSelectedStatus("Bomber", selectedCount);
		else if (selectedSprite->GetState() & BUILDER)
			statusPanel->SetSelectedStatus("Builder", selectedCount);
		else if (selectedSprite->GetState() & DIGGER)
			statusPanel->SetSelectedStatus("Digger", selectedCount);
		else if (selectedSprite->GetState() & MINER)
			statusPanel->SetSelectedStatus("Miner", selectedCount);
		else if (selectedSprite->GetState() & BASHER)
			statusPanel->SetSelectedStatus("Basher", selectedCount);
		else if (selectedSprite->GetState() & BLOCKER)
			statusPanel->SetSelectedStatus("Blocker", selectedCount);
		else if (!inspectLemmingMode) {
			statusPanel->SetSelectedStatus("", 0);
		}
	} else {
		if (!inspectLemmingMode) {
			statusPanel->SetSelectedStatus("", 0);
		}
	}
}

bool Game::SetMousePosition(int x, int y, long leftButtonDown,
		long rightButtonDown, long lbuttonHeld) {
	bool updateRequired = false;
	mouseX = x;
	mouseY = y;

	if (x < ((gameType != StatusPanel::SinglePlayer) ? 64 : 2)) {
		statusPanel->viewport.x += -(rightButtonDown ? 8 : 1);
		if (statusPanel->viewport.x <= 0)
			statusPanel->viewport.x = 0;
		else
			updateRequired = true;
	} else if (x > screenWidth * 2 - 2) {
		statusPanel->viewport.x -= -(rightButtonDown ? 8 : 1);
		if (statusPanel->viewport.x > currentLevel->GetBackgroundWidth()
				- (screenWidth * zoom)) {
			statusPanel->viewport.x = currentLevel->GetBackgroundWidth()
					- screenWidth * zoom;
		} else {
			updateRequired = true;
		}
	}

	if (lbuttonHeld && gameType == StatusPanel::SinglePlayer) {
		if (mouseY > statusPanel->ypos) {
			unsigned int rr = currentLevel->GetReleaseRate();
			if (mouseX > statusPanel->iconWidth+6 && mouseX <= statusPanel->iconWidth+6 + statusPanel->iconWidth) {
				if (rr < 99) {
					currentLevel->SetReleaseRate(rr + 1);
					updateRequired = true;
				}
			} else if (mouseX > 8 && mouseX <= 8 + 20) {
				if (rr > gameData.minReleaseRate) {
					currentLevel->SetReleaseRate(rr - 1);
					updateRequired = true;
				}
			}
		}
	}

	if (lbuttonHeld) {
		statusPanel->UpdateOverviewOffset(x, y);
	}

	currentLevel->SetViewPort(&statusPanel->viewport);

	return updateRequired;
}

void Game::Scroll(int amount) {
	statusPanel->Scroll(amount);
}

void Game::SelectItem(int direction) {
	if (direction < 0) {
		statusPanel->MoveItemLeft();
	} else {
		statusPanel->MoveItemRight();
	}
}

void Game::HandleKey(int key) {
	switch (key) {
	case (SDLK_UP):
#ifdef DEBUG
		sprites->TestSprite(1);
#endif
		break;

	case (SDLK_DOWN):
#ifdef DEBUG
		sprites->TestSprite(-1);
#endif
		break;

	case (SDLK_RIGHT):
#ifdef DEBUG
		sprites->TestFrame(1);
#endif
		break;

	case (SDLK_LEFT):
#ifdef DEBUG
		sprites->TestFrame(-1);
#endif
		break;

	case (SDLK_m):
		effects_only = !effects_only;
		break;

	case (SDLK_p):
		if (gameType == StatusPanel::SinglePlayer)
			Pause();
		break;

	case (SDLK_d):
#if DEBUG
		debugMode++;
		if (debugMode == 4)
		debugMode = 0;
#endif
		//currentLevel->debug = debugMode>0;
		break;
	case (SDLK_a):
#if DEBUG
		AddNewLemming(mouseX, mouseY);
#endif
		break;

	case (SDLK_g):
#if DEBUG
		//for (unsigned int nf = 0; nf < arAllGameChanges.size(); nf++) {
		//	printf("%d, %d, %d\n", arAllGameChanges[nf].frameIndex,
		//			arAllGameChanges[nf].index, arAllGameChanges[nf].new_state);
		//}
#endif
		break;
	case (SDLK_h):
#if DEBUG
		//PlaybackGame();
#endif
		break;

	case (SDLK_j):
#if DEBUG
		//PlaybackGame();
		StartPlayback();
#endif
		break;
	}

}

int Game::GetPercentDone() {
	return (gameData.numHome * 100) / currentLevel->GetNumLemmings();
}

/*
 * Draw everything on the screen.
 */
void Game::Draw(SDL_Surface *screen) {
	//Clear the screen
	SDL_FillRect(screen, screen_size, BACKGROUND_COLOUR);

	currentLevel->DrawExitsEntrancesWater(screen, NULL);
	utility->apply_surface(0, 0, currentLevel->background, screen,
			&statusPanel->viewport);
	currentLevel->RenderAllOneWayIndicators(screen);

	statusPanel->Draw(screen);
	DrawLemmings(screen);
	statusPanel->DrawTotals(screen);

#if DEBUG
	if(debugMode == 2) {
		currentSinglePlayerLevel->DrawDebugStuff(screen);
		sprites->DrawTestAnimationBlocks(screen);
	}
	else if(debugMode == 3)
	sprites->DrawAnimations(screen);

	if (selectedSprite != NULL && debugMode)
	utility->DrawBox(screen, selectedSprite->position->x
			- statusPanel->viewport.x - 3, selectedSprite->position->y
			- statusPanel->viewport.y, 10, 12);
#endif
}
void Game::DrawCursor(SDL_Surface* screen) {
	if (selectedSprite == NULL || inspectLemmingMode) {//just draw cross-hair
		utility->apply_surface(mouseX - sprites->crossHair.offset.x, mouseY
				- sprites->crossHair.offset.y, sprites->cursors, screen,
				&sprites->crossHair.clip);
	} else {//draw selected 'box'
		utility->apply_surface(mouseX - sprites->LemmingSelected.offset.x,
				mouseY - sprites->LemmingSelected.offset.y, sprites->cursors,
				screen, &sprites->LemmingSelected.clip);

		/*	if(inspectLemmingMode){
		 int xx = selectedSprite->position->x;
		 int yy = selectedSprite->position->y;
		 utility->apply_surface(sprites->LemmingSelected.offset.x - xx,
		 sprites->LemmingSelected.offset.y - yy, sprites->cursors,
		 screen, &sprites->LemmingSelected.clip);
		 }*/
	}
}

void Game::DrawLemmings(SDL_Surface* screen) {
	Lemming* s;
	StatusPanel::DrawMode drawtype = StatusPanel::Nothing;
	int state;
	SpriteStore::AnimObject *animFrame;
	for (int n = 0; n < max_num_lemmings; n++) {
		drawtype = StatusPanel::Nothing;
		s = ulemmings.lemmings[n];
		state = s->GetState();
		if (s->GetAliveState() && !(s->GetState() & ARRIVED_HOME)) {
			//first get the correct clip/sprite
			animFrame = NULL;
			if (state & WALKING) {
				if (state & GOING_RIGHT) {
					animFrame = &sprites->walkerRight[s->GetFrame()];
				} else {
					animFrame = &sprites->walkerLeft[s->GetFrame()];
				}
			} else if (state & DIGGER) {
				animFrame = &sprites->diggerDowns[s->GetFrame()];
				drawtype = StatusPanel::NoPixel;
			} else if (state & BUILDER) {
				drawtype = StatusPanel::Pixel;
				if (state & GOING_RIGHT) {
					animFrame = &sprites->builderRights[s->GetFrame()];
				} else {
					animFrame = &sprites->builderLefts[s->GetFrame()];
				}
				if (s->GetBuilderRunOut()) {
					samples->PlaySample(TING);
				}
			} else if (state & NOW_CLIMBING) {
				if (state & GOING_RIGHT) {
					animFrame = &sprites->climberRights[s->GetFrame()];
				} else {
					animFrame = &sprites->climberLefts[s->GetFrame()];
				}
			} else if (state & STOPPED_CLIMBING) {
				if (state & GOING_RIGHT) {
					animFrame = &sprites->stoppedClimbingRights[s->GetFrame()];
				} else {
					animFrame = &sprites->stoppedClimbingLefts[s->GetFrame()];
				}
			} else if (state & BLOCKER) {
				animFrame = &sprites->blockers[s->GetFrame()];
#ifdef DEBUG
				if (debugMode)
				utility->DrawBox(screen, s->position->x-3 - statusPanel->viewport.x,
						s->position->y - statusPanel->viewport.y,
						6, s->GetHeight()+4);
#endif
			} else if (state & FALLING) {
				if ((state & FLOATER) && s->FallingFarEnoughToOpenBrolly()) {
					if (state & GOING_RIGHT) {
						if (state & OPENING_BROLLY) {
							animFrame
									= &sprites->openBrollyRight[s->GetFrame()];
						} else {
							animFrame = &sprites->floaterRight[s->GetFrame()];
						}
					} else {
						if (state & OPENING_BROLLY) {
							animFrame = &sprites->openBrollyLeft[s->GetFrame()];
						} else {
							animFrame = &sprites->floaterLeft[s->GetFrame()];
						}
					}
				} else {
					if (state & GOING_RIGHT) {
						animFrame = &sprites->fallerRight[s->GetFrame()];
					} else {
						animFrame = &sprites->fallerLeft[s->GetFrame()];
					}
				}
			} else if (state & BASHER) {
				drawtype = StatusPanel::NoPixel;
				if (state & GOING_RIGHT) {
					animFrame = &sprites->basherRights[s->GetFrame()];
				} else {
					animFrame = &sprites->basherLefts[s->GetFrame()];
				}
			} else if (state & MINER) {
				drawtype = StatusPanel::NoPixel;
				if (state & GOING_RIGHT) {
					animFrame = &sprites->minerRights[s->GetFrame()];
				} else {
					animFrame = &sprites->minerLefts[s->GetFrame()];
				}
			} else if (state & DONE_BUILDING) {
				if (state & GOING_RIGHT) {
					animFrame = &sprites->doneBuildingRights[s->GetFrame()];
				} else {
					animFrame = &sprites->doneBuildingLefts[s->GetFrame()];
				}
			} else if (state & SPLATTED) {
				if (s->GetFrame() == 0)
					samples->PlaySample(SPLAT);
				animFrame = &sprites->splatters[s->GetFrame()];
			} else if ((state & NUKE)) {
				animFrame = &sprites->nukes[s->GetFrame()];
			} else if (state & GOING_HOME) {
				if (state & GOING_RIGHT) {
					animFrame = &sprites->goingHomeRight[s->GetFrame()];
				} else {
					animFrame = &sprites->goingHomeLeft[s->GetFrame()];
				}
			} else if (state & DROWNER) {
				animFrame = &sprites->drowners[s->GetFrame()];
				if (s->GetFrame() == 0)
					samples->PlaySample(SPLASH);
			} else if (state & BURNER) {
				animFrame = &sprites->burners[s->GetFrame()];
				if (s->GetFrame() == 0)
					samples->PlaySample(FIRE);
			}

#ifdef DEBUG
			//debug point
			if (debugMode)
			utility->DrawBox(screen, s->position->x - statusPanel->viewport.x,
					s->position->y + s->GetHeight() - 2 - statusPanel->viewport.y,
					2, 2);
#endif
			if (animFrame != NULL) {

				//finally actually draw the sprite frame:
				int tx = (s->position->x - statusPanel->viewport.x)
						+ animFrame->offset.x * zoom;
				int ty = (s->position->y - statusPanel->viewport.y)
						+ animFrame->offset.y * zoom;
				if (tx < screen_size->w && tx > (gameType
						!= StatusPanel::SinglePlayer ? 58 : 0) - s->GetWidth()
						&& ty > 0 && ty < screen_size->h) {
					if (s->GetPlayerNum() == 1 || gameType
							== StatusPanel::SinglePlayer) {
						//draw a blue lemming!
						utility->apply_surface_zoomed(tx, ty,
								sprites->player1Sprites, screen,
								&animFrame->clip, zoom);
					} else {
						//draw a green lemming!
						utility->apply_surface_zoomed(tx, ty,
								sprites->player2Sprites, screen,
								&animFrame->clip, zoom);
					}

					//Draw selection around selected lemming if inspect mode:
					if (s == selectedSprite && inspectLemmingMode) {
						utility->apply_surface(
								tx - 4 /*+ sprites->LemmingSelected.offset.x*/,
								ty - 2 /*+ sprites->LemmingSelected.offset.y*/,
								sprites->cursors, screen,
								&sprites->LemmingSelected.clip);
					}
				}

				statusPanel->DrawMarker(s->position->x, s->position->y, screen,
						drawtype);
			}

			//if the lemming is moribund (dead or dying, in this case dying)
			short timeToDie = s->CountingDown();
			if ((s->GetRawNukeCount() > 1) && timeToDie != -2 && !(state
					& EXPLODING)) {
				//draw a countdown timer
				SDL_Rect* timeRect = &sprites->countDown[timeToDie].clip;
				utility->apply_surface_zoomed(s->position->x
						- statusPanel->viewport.x
						+ sprites->countDown[timeToDie].offset.x * zoom,
						s->position->y - statusPanel->viewport.y - (6 * zoom),
						sprites->player1Sprites, screen, timeRect, zoom);
			}

			if (debugMode)
				utility->DrawBox(screen, s->position->x
						- statusPanel->viewport.x, s->position->y
						+ s->GetHeight() - 2 - statusPanel->viewport.y, 2, 2);

		}

		if (state & EXPLODING) {
			if (s->DrawExplosion(screen, &statusPanel->viewport, screenWidth*zoom, screenHeight*zoom) == 1) {
				utility->apply_surface_zoomed(s->position->x - (16 * zoom)
						- statusPanel->viewport.x, s->position->y - (10 * zoom)
						- statusPanel->viewport.y, sprites->player1Sprites,
						screen, &sprites->bang.clip, zoom);
			}
		}
	}
}
void Game::AddNewLemming(int mx, int my) {
	ulemmings.lemmings[gameData.blueOut]->SetAlive(
			mx + statusPanel->viewport.x, (my + statusPanel->viewport.y));
	ulemmings.lemmings[gameData.blueOut]->SetPlayerNum(1);
	gameData.blueOut++;
}
/*
 * Adds a new Lemming.
 */
void Game::AddNewLemming() {
	bool green, blue;
	green = blue = false;
	int lemIndex = -1;

	//is this player a blue (player 1) or green (player 2)
	if (gameType != StatusPanel::SinglePlayer) {
		if (gameData.greenOut < greenGameMaxLems && gameData.blueOut
				< blueGameMaxLems) {
			if ((gameData.numberReleased % 2)) {//green player
				green = true;
				lemIndex = gameData.greenOut + 80;
			} else {
				blue = true;
				lemIndex = gameData.blueOut;
			}
		} else {
			if ((gameData.numberReleased % 2) && greenGameMaxLems
					> gameData.greenOut) {
				green = true;
				lemIndex = gameData.greenOut + 80;
			} else if (greenGameMaxLems > gameData.blueOut) {
				blue = true;
				lemIndex = gameData.blueOut;
			}
		}
	} else {
		blue = true;
		lemIndex = gameData.blueOut;
	}

	ulemmings.lemmings[lemIndex]->SetAlive(currentLevel->GetEntranceX(
			gameData.releasePoint), currentLevel->GetEntranceY(
			gameData.releasePoint) - 16);

	if (green) {
		ulemmings.lemmings[lemIndex]->SetPlayerNum(2);
		ulemmings.lemmings[lemIndex]->ChangeDirection();
		gameData.greenOut++;
	} else if (blue) {
		ulemmings.lemmings[lemIndex]->SetPlayerNum(1);
		gameData.blueOut++;
	}

	if (gameType != StatusPanel::SinglePlayer) {
		if (green) {
			if (twoplayerhack->green_heading_right == 1) {
				ulemmings.lemmings[lemIndex]->ChangeDirection();
			}
		} else {
			if (twoplayerhack->blue_heading_right == 1) {
				ulemmings.lemmings[lemIndex]->ChangeDirection();
			}
		}
	}

	gameData.numberReleased++;
	gameData.releasePoint++;
}

int Game::TimeRemaining() {
	return timeRemaining;
}

void Game::SetMusicOn() {
	effects_only = false;
	//musicwason = false;
}

void Game::SetMusicOff() {
	effects_only = true;
}

void Game::SetAllSoundOff() {
	//TODO: Implement this function!
}

/*
 * Test to see if the game is complete.
 */
bool Game::IsGameComplete() {

	if (gameCountDown != -1) {
		gameCountDown--;
		if (gameCountDown == 1) {
			return true;
		}
	} else {
		if (timeRemaining <= 0 || LemmingsSavedPercent() == 100
				|| (gameData.numberReleased == currentLevel->GetNumLemmings()
						&& gameData.totalOut == 0) || (nukeAll
				&& gameData.totalOut == 0)) {
			if (nukeAll) {
				gameCountDown = 250;
			} else {
				gameCountDown = 25;
			}
			return false;
		}
	}

	return false;
}

int Game::LemmingsSavedPercent() {
	return ((gameData.numHome * 100) / (currentLevel->GetNumLemmings()));
}

bool Game::WasGameNuked() {
	return nukeAll && (gameData.nukeCount > 100 && gameData.totalOut == 0);
}

/*
 * Returns true if the game was won, false otherwise.
 */
bool Game::WasGameWon() {
	if (gameType == StatusPanel::SinglePlayer)
		return currentLevel->GetLemmingsToBeSaved() <= gameData.numHome;
	else {
		if (BlueHome() != GreenHome()) {
			if (gameType == StatusPanel::Server)
				return BlueHome() > GreenHome();
			else {
				return GreenHome() > BlueHome();
			}
		} else {
			//game draw
			return false;
		}
	}
}

int Game::AnimateLemming(Lemming* s) {
	int gone_home = -1;
	if (s->GetAliveState() && !(s->GetState() & ARRIVED_HOME)) {
		if (!(s->GetState() & EXPLODING)) {
			gameData.totalOut++;
			if (s->GetPlayerNum() == 1 && gameType == StatusPanel::Server)
				gameData.thisTotalOut++;
			else if (s->GetPlayerNum() == 2 && gameType == StatusPanel::Client)
				gameData.thisTotalOut++;
		}
		s->AnimateFrame();
		s->Move(currentLevel->GetBackground());

		//now check the lemming hasn't gone outside of the screen
		if (s->position->x >= LEVEL_X_SIZE * zoom || s->position->x < 0
				|| s->position->y >= (LEVEL_Y_SIZE - 10) * zoom
				|| s->position->y < 0) {
			samples->PlaySample(DIE);
			s->SetDead();
		} else {
			for (int m = 0; m < max_num_lemmings; m++) {
				if (ulemmings.lemmings[m] != s) {
					s->CheckForBlocker(ulemmings.lemmings[m], m);
				}
			}

			int state = s->GetState();
			int x = s->position->x;
			int y = s->position->y;

			if (nukeAll && gameData.nukeCount >= 2) {
				//don't nuke them all at the same time,
				//it looks much better if they go off at different times.
				if (!(s->GetState() & GOING_HOME) && !(s->GetState() & NUKE)
						&& !(s->GetState() & EXPLODING) && !(s->GetState()
						& DROWNER) && !(s->GetState() & BURNER)) {
					s->SetToNuke();
					gameData.nukeCount = 1;
				}
			}

			//Now check if the lemming has reached the exit
			if (!(s->GetState() & GOING_HOME) && (s->GetState() != DEAD)
					&& !(s->GetState() & EXPLODING) && !(s->GetState() & ARRIVED_HOME) ) {
				//gone_home = currentLevel->IsPointInHome(s->position->x, s->position->y);
				if (currentLevel->IsPointInHome(x, y) != -1) {
					//but a blue entrance or a green one?
					s->SetGoingHome();
					samples->PlaySample(BOING);
				}
			}

			if (s->GetState() & ARRIVED_HOME) {
				//is he home yet?
				gone_home = currentLevel->IsPointInHome(s->position->x,
						s->position->y);
				if (gone_home != -1) {
					//but a blue entrance or a green one?
					//if (gone_home % 2 == 0) {
					if (currentLevel->IsBlueHome(gone_home)) {
						gone_home = StatusPanel::Server;
					} else {
						gone_home = StatusPanel::Client;
					}
					gameData.numHome++;
				}
			}

			if (currentLevel->IsPointInTrap(x, y)) {
				s->SetDead();
			} else if (currentLevel->IsPointInHazard(x, y)
					&& !(state & DROWNER) && !(state & BURNER) && (state
					!= DEAD)) {
				s->SetBurning();
			} else if (!(state & DROWNER) && (state != DEAD)) {
				if (currentLevel->IsPointInWater(x, y))
					s->SetDrowning();
			}

			//now Stuff that affects the background
			//check for miners,builders,diggers,bashers,bombers
			if (state & DIGGER) {
				DoDigAction(s);
			} else if (state & MINER) {
				DoMinerAction(s);
			} else if ((state & BASHER)) {
				DoBasherAction(s);
			} else if (state & NUKE && (!nukeAll && s->GetRawNukeCount() == 1)) {
				samples->PlaySample(OHNO);
			} else if ((state & EXPLODING) && s->ShallIDrawExplosion()) {
				DoExplosion(s);
			} else if ((state & BUILDER) && s->ShallIBuildBlock()) {
				DoBuildAction(s);
			}
		}
	}
	return gone_home;
}

void Game::DoDigAction(Lemming* s) {
	if (!currentLevel->IsPointMetalArea(s->position->x, s->position->y)) {
		//where are we digging/affecting backgrund? Lemming or here!?
		if (s->GetFrame() == 0 || s->GetFrame() == 8 || s->GetFrame() == 15) {

			/*if(gameType != StatusPanel::SinglePlayer){
			 ImageBlock* ib = &arRebuildSurfaces[rebuildSurfaceCount];

			 if(ib->image != NULL) SDL_FreeSurface(ib->image);

			 //store this graphic area
			 ib->x = s->position->x - (4*zoom);
			 ib->y = s->position->y + (7*zoom);
			 ib->image = SDL_CreateRGBSurface(currentLevel->GetBackground()->flags,
			 (9*zoom),
			 (3*zoom),
			 currentLevel->GetBackground()->format->BitsPerPixel,
			 currentLevel->GetBackground()->format->Rmask,
			 currentLevel->GetBackground()->format->Gmask,
			 currentLevel->GetBackground()->format->Bmask,
			 currentLevel->GetBackground()->format->Amask);

			 ib->image->format->colorkey = SDL_MapRGB(ib->image->format, 255,128, 0);

			 //now copy the pixels across
			 utility->CopyArea(currentLevel->GetBackground(),ib->image,ib->x,ib->y);

			 //up the count!
			 rebuildSurfaceCount++;
			 }*/

			//now actually draw the thing
			SDL_Rect block;
			block.w = (9 * zoom);
			block.h = (3 * zoom);
			block.x = s->position->x - (4 * zoom);
			block.y = s->position->y + (7 * zoom);
			SDL_FillRect(currentLevel->GetBackground(), &block, BLACK_COLOUR);
		}
	} else {
		samples->PlaySample(CHINK);
		s->SetToWalker();
	}
}

void Game::DoMinerAction(Lemming* s) {
	if (s->GetState() & GOING_RIGHT) {
		if (!currentLevel->IsPointMetalArea(s->position->x + 10, s->position->y)) {
			if (s->Mined()) {
				//draw a black blob in the right place...
				int f = s->GetFrame();
				if (f <= 3) {
					/*if(gameType != StatusPanel::SinglePlayer){
					 ImageBlock* ib = &arRebuildSurfaces[rebuildSurfaceCount];

					 if(ib->image != NULL) SDL_FreeSurface(ib->image);

					 //store this graphic area
					 ib->x = s->position->x + sprites->bashRightMasks[f].offset.x * zoom;
					 ib->y = s->position->y + sprites->bashRightMasks[f].offset.y * zoom;
					 ib->image = SDL_CreateRGBSurface(currentLevel->GetBackground()->flags,
					 sprites->bashRightMasks[f].clip.w*zoom,
					 sprites->bashRightMasks[f].clip.h*zoom,
					 currentLevel->GetBackground()->format->BitsPerPixel,
					 currentLevel->GetBackground()->format->Rmask,
					 currentLevel->GetBackground()->format->Gmask,
					 currentLevel->GetBackground()->format->Bmask,
					 currentLevel->GetBackground()->format->Amask);
					 ib->image->format->colorkey = SDL_MapRGB(ib->image->format, 255,128, 0);
					 //now copy the pixels across
					 utility->CopyArea(currentLevel->GetBackground(),ib->image,ib->x,ib->y);

					 //up the count!
					 rebuildSurfaceCount++;
					 }*/

					utility->apply_mask(s->position->x
							+ sprites->bashRightMasks[f].offset.x * zoom,
							s->position->y
									+ sprites->bashRightMasks[f].offset.y
											* zoom, sprites->player1Sprites,
							currentLevel->GetBackground(),
							&sprites->bashRightMasks[f].clip, MASK_COLOUR, zoom);
				}
			}
		} else {
			samples->PlaySample(CHINK);
			s->SetToWalker();
		}
	} else if (!currentLevel->IsPointMetalArea(s->position->x - 8,
			s->position->y)) {

		if (s->Mined()) {
			//draw a black blob in the right place...
			int f = s->GetFrame();
			if (f <= 3) {
				/*if(gameType != StatusPanel::SinglePlayer){
				 ImageBlock* ib = &arRebuildSurfaces[rebuildSurfaceCount];

				 if(ib->image != NULL) SDL_FreeSurface(ib->image);

				 //store this graphic area
				 ib->x = s->position->x + sprites->bashLeftMasks[f].offset.x * zoom;
				 ib->y = s->position->y + sprites->bashLeftMasks[f].offset.y * zoom;
				 ib->image = SDL_CreateRGBSurface(currentLevel->GetBackground()->flags,
				 sprites->bashLeftMasks[f].clip.w*zoom,
				 sprites->bashLeftMasks[f].clip.h*zoom,
				 currentLevel->GetBackground()->format->BitsPerPixel,
				 currentLevel->GetBackground()->format->Rmask,
				 currentLevel->GetBackground()->format->Gmask,
				 currentLevel->GetBackground()->format->Bmask,
				 currentLevel->GetBackground()->format->Amask);
				 ib->image->format->colorkey = SDL_MapRGB(ib->image->format, 255,128, 0);
				 //now copy the pixels across
				 utility->CopyArea(currentLevel->GetBackground(),ib->image,ib->x,ib->y);

				 //up the count!
				 rebuildSurfaceCount++;
				 }*/

				utility->apply_mask(s->position->x
						+ sprites->bashLeftMasks[f].offset.x * zoom,
						s->position->y + sprites->bashLeftMasks[f].offset.y
								* zoom, sprites->player1Sprites,
						currentLevel->GetBackground(),
						&sprites->bashLeftMasks[f].clip, MASK_COLOUR, zoom);
			}
		}
	} else {
		samples->PlaySample(CHINK);
		s->SetToWalker();
	}
}

void Game::DoBasherAction(Lemming* s) {
	SpriteStore::AnimObject* obj = NULL;
	if ((s->GetState() & GOING_RIGHT)) {
		if (!currentLevel->IsPointMetalArea(s->position->x + 2, s->position->y)
				&& !currentLevel->IsPointOneWayArea(s->position->x + 2,
						s->position->y - 4, 1)) {
			switch (s->GetFrame()) {
			case (3):
			case (19):
				obj = &sprites->bashRightMasks[1];
				break;
			case (4):
			case (20):
				obj = &sprites->bashRightMasks[2];
				break;
			case (5):
			case (21):
				obj = &sprites->bashRightMasks[3];
				break;
			}
			if (obj != NULL) {
				/*if(gameType != StatusPanel::SinglePlayer){
				 ImageBlock* ib = &arRebuildSurfaces[rebuildSurfaceCount];

				 if(ib->image != NULL) SDL_FreeSurface(ib->image);

				 //store this graphic area
				 ib->x = s->position->x;
				 ib->y = s->position->y + zoom;
				 ib->image = SDL_CreateRGBSurface(currentLevel->GetBackground()->flags,
				 obj->clip.w*zoom,
				 obj->clip.h*zoom,
				 currentLevel->GetBackground()->format->BitsPerPixel,
				 currentLevel->GetBackground()->format->Rmask,
				 currentLevel->GetBackground()->format->Gmask,
				 currentLevel->GetBackground()->format->Bmask,
				 currentLevel->GetBackground()->format->Amask);
				 ib->image->format->colorkey = SDL_MapRGB(ib->image->format, 255,128, 0);

				 //now copy the pixels across
				 utility->CopyArea(currentLevel->GetBackground(),ib->image,ib->x,ib->y);

				 //up the count!
				 rebuildSurfaceCount++;
				 }*/

				utility->apply_mask(s->position->x, s->position->y + zoom,
						sprites->player1Sprites, currentLevel->GetBackground(),
						&obj->clip, MASK_COLOUR, zoom);
			}
		} else {
			samples->PlaySample(CHINK);
			s->SetToWalker();
		}
	} else {//GOING LEFT
		if (!currentLevel->IsPointMetalArea(s->position->x - 2, s->position->y)
				&& !currentLevel->IsPointOneWayArea(s->position->x - 2,
						s->position->y - 4, 0)) {
			switch (s->GetFrame()) {
			case (3):
			case (19):
				obj = &sprites->bashLeftMasks[1];
				break;
			case (4):
			case (20):
				obj = &sprites->bashLeftMasks[2];
				break;
			case (5):
			case (21):
				obj = &sprites->bashLeftMasks[3];
				break;
			}
			if (obj != NULL) {
				/*if(gameType != StatusPanel::SinglePlayer){
				 ImageBlock* ib = &arRebuildSurfaces[rebuildSurfaceCount];

				 if(ib->image != NULL) SDL_FreeSurface(ib->image);

				 //store this graphic area
				 ib->x = s->position->x - 7*zoom - 1*zoom;
				 ib->y = s->position->y + zoom;
				 ib->image = SDL_CreateRGBSurface(currentLevel->GetBackground()->flags,
				 obj->clip.w*zoom,
				 obj->clip.h*zoom,
				 currentLevel->GetBackground()->format->BitsPerPixel,
				 currentLevel->GetBackground()->format->Rmask,
				 currentLevel->GetBackground()->format->Gmask,
				 currentLevel->GetBackground()->format->Bmask,
				 currentLevel->GetBackground()->format->Amask);

				 ib->image->format->colorkey = SDL_MapRGB(ib->image->format, 255,128, 0);

				 //now copy the pixels across
				 utility->CopyArea(currentLevel->GetBackground(),ib->image,ib->x,ib->y);

				 //up the count!
				 rebuildSurfaceCount++;
				 }*/

				utility->apply_mask(s->position->x - 7 * zoom - 1 * zoom,
						s->position->y + zoom, sprites->player1Sprites,
						currentLevel->GetBackground(), &obj->clip, MASK_COLOUR,
						zoom);
			}
		} else {
			samples->PlaySample(CHINK);
			s->SetToWalker();
		}
	}
}

void Game::DoExplosion(Lemming* s) {
	//blow a hole in the landscape
	if (!currentLevel->IsPointExpandedMetalArea(s->position->x - 8,
			s->position->y - 8)) {

		/*if(gameType != StatusPanel::SinglePlayer){
		 ImageBlock* ib = &arRebuildSurfaces[rebuildSurfaceCount];

		 if(ib->image != NULL) SDL_FreeSurface(ib->image);

		 //store this graphic area
		 ib->x = s->position->x - (8*zoom);
		 ib->y = s->position->y - (8*zoom);
		 ib->image = SDL_CreateRGBSurface(currentLevel->GetBackground()->flags,
		 sprites->explosion.clip.w*zoom,
		 sprites->explosion.clip.h*zoom,
		 currentLevel->GetBackground()->format->BitsPerPixel,
		 currentLevel->GetBackground()->format->Rmask,
		 currentLevel->GetBackground()->format->Gmask,
		 currentLevel->GetBackground()->format->Bmask,
		 currentLevel->GetBackground()->format->Amask);

		 ib->image->format->colorkey = SDL_MapRGB(ib->image->format, 255,128, 0);

		 //now copy the pixels across
		 utility->CopyArea(currentLevel->GetBackground(),ib->image,ib->x,ib->y);

		 //up the count!
		 rebuildSurfaceCount++;
		 }*/

		utility->apply_mask(s->position->x - (8 * zoom), s->position->y - (8
				* zoom), sprites->player1Sprites,
				currentLevel->GetBackground(), &sprites->explosion.clip,
				MASK_COLOUR, zoom);
	}

	samples->PlaySample(EXPLODE);
}

void Game::DoBuildAction(Lemming* s) {
	SDL_Rect block;
	block.w = s->GetWidth();
	block.h = zoom;

	block.y = s->position->y + s->GetHeight();

	if (s->GetState() & GOING_LEFT)
		block.x = s->position->x - (5 * zoom);
	else
		block.x = s->position->x + (1 * zoom);

	/*if(gameType != StatusPanel::SinglePlayer){
	 ImageBlock* ib = &arRebuildSurfaces[rebuildSurfaceCount];

	 if(ib->image != NULL) SDL_FreeSurface(ib->image);

	 //store this graphic area
	 ib->x = block.x;
	 ib->y = block.y;
	 ib->image = SDL_CreateRGBSurface(currentLevel->GetBackground()->flags,
	 s->GetWidth(),
	 zoom,
	 currentLevel->GetBackground()->format->BitsPerPixel,
	 currentLevel->GetBackground()->format->Rmask,
	 currentLevel->GetBackground()->format->Gmask,
	 currentLevel->GetBackground()->format->Bmask,
	 currentLevel->GetBackground()->format->Amask);
	 //now copy the pixels across
	 utility->CopyArea(currentLevel->GetBackground(),ib->image,ib->x,ib->y);

	 //up the count!
	 rebuildSurfaceCount++;
	 }*/

	//draw the actual block
	SDL_FillRect(currentLevel->GetBackground(), &block, SDL_MapRGB(
			currentLevel->GetBackground()->format, 0xFF, 0xFF, 0xFF));

}

bool Game::DoLemmingAction(DataBuffer::NetData* lemData, Lemming* incomingLem,
		int lemIndex) {
	bool actionDone = false;

	if (lemData->index != INVALID_INDEX && (int) lemData->index == lemIndex) {
		switch (lemData->new_state) {
		case (7):
			actionDone = incomingLem->SetToDigger();
			break;
		case (6):
			actionDone = incomingLem->SetToMiner();
			break;
		case (5):
			actionDone = incomingLem->SetToBasher();
			break;
		case (4):
			actionDone = incomingLem->SetToBuilder();
			break;
		case (3):
			actionDone = incomingLem->SetToBlocker();
			break;
		case (2):
			actionDone = incomingLem->SetToNuke();
			break;
		case (1):
			actionDone = incomingLem->SetToFloater();
			break;
		case (0):
			actionDone = incomingLem->SetToClimber();
			break;

		default:
			//printf("!!!!!!!!!!BAD LEMMING STATE SENT!!!!!!!!!!!! index: %d state: %d\n", lemData->index, lemData->new_state);
			break;
		}
	}

	//if(storeChanges && actionDone){
	//	//store the change in out ever increasingly large array:
	//	DataBuffer::NetData d;
	//	d.frameIndex = frameIndex;
	//	printf("frameindex: %d\n", frameIndex);
	//	d.index = lemData->index;
	//	d.new_state = lemData->new_state;
	//}

	return actionDone;
}

StatusPanel::GameType Game::GetGameType() {
	return gameType;
}

int Game::BlueHome() {
	return statusPanel->BlueHome();
}

int Game::GreenHome() {
	return statusPanel->GreenHome();
}

void Game::ResetMultiplayer() {
	blueGamesWon = greenGamesWon = 0;
	blueGameMaxLems = greenGameMaxLems = min_multiplay_lemmings;
}

void Game::CloseConnection() {
	dataBufferer->CloseConnection();
}

/* 
 * Animates the frame or in the case of being a 'client' gets the latest positions
 * from the server
 */
bool Game::AnimateFrame() {
	bool retVal = true;
	if (gameType == StatusPanel::SinglePlayer) {
		ProcessFrame(&myGameChange, NULL);
	} else {

		// This way should be simpler
		DataBuffer::NetData incomingGameChange;
		// get and send the latest moves
		(void) dataBufferer->RequestForData(&myGameChange, &incomingGameChange);
		// use those moves to process the next frame
		ProcessFrame(&myGameChange, &incomingGameChange);

		retVal = !IsGameComplete();
	}

	//clear out the change so it doesn't happen again...
	//myGameChange.frameIndex = 0;
	myGameChange.index = INVALID_INDEX;
	myGameChange.new_state = INVALID_NEW_STATE;

	if (!retVal) {
		//ARRGGHH!!! Why can't I disconnect!!!!
		dataBufferer->ResetConnection();
		delete (dataBufferer);
		dataBufferer = new DataBuffer();
	}

	return retVal;
}

/*
 * Animates the frame or in the case of being a 'client' gets the latest positions
 * from the server
 */
void Game::ProcessFrame(DataBuffer::NetData* player1,
		DataBuffer::NetData* player2) {

	if (!pause) {
		frameIndex++;
		//animate moving stuff:
		currentLevel->Animate();

		int tot;
		if (gameType == StatusPanel::SinglePlayer) {
			tot = gameData.totalOut;
		} else {
			tot = gameData.thisTotalOut;
		}

		//ensure status panel shows the right stuff:
		statusPanel->SetGameStats(timeRemaining, tot, (gameData.numHome * 100)
				/ currentLevel->GetNumLemmings(),
				(int) currentLevel->GetReleaseRate(), currentLevel->Tools(),
				gameData.minReleaseRate);

		//check if we need to start the game proper:
		if (gameData.ticksSinceGameStart < GAME_START_PAUSE) {
			if (gameData.ticksSinceGameStart == 0) {
				samples->PlaySample(LETSGO);
			} else if (gameData.ticksSinceGameStart > 18) {
				currentLevel->AnimateEntrance();
			} else if (gameData.ticksSinceGameStart == 18) {
				samples->PlaySample(DOOR);
			}
			gameData.ticksSinceGameStart++;
		} else {
			//now start the actual game:
			if (sound_enabled) {
				if (!effects_only) {
					if (!music_now_on) {
						if (Mix_PlayMusic(levelTune, -1) != -1) {
							music_now_on = true;
						}
					}
				} else if (music_now_on) {
					Mix_FadeOutMusic(500);
					printf("fading out music\n");
					music_now_on = false;
					musicwason = true;
				}
			}

			//only count-down if the game is NOT 2 player.l
			if (++fpsCount >= fps && (gameType
					== StatusPanel::SinglePlayer)) {
				timeRemaining--;
				fpsCount = 1;
			}

			if (!nukeAll) {
				if (++gameData.releaseCounter >= (60
						- ((int) currentLevel->GetReleaseRate() * 60 / 105))
						&& gameData.numberReleased
								< currentLevel->GetNumLemmings()) {
					if (!debugMode && GetPercentDone() < 100) {
						AddNewLemming();
					}
					gameData.releaseCounter = 0;
				}

				if (blueNuked && greenNuked) {
					gameData.nukeCount = 0;
					samples->PlaySample(OHNO);
					nukeAll = true;
				} else if (player1 != NULL && player1->new_state == 8) {
					if (gameType == StatusPanel::Client) {
						printf("blue nuked\n");
						blueNuked = true;
					} else {
						printf("green nuked\n");
						greenNuked = true;
					}
				} else if (player2 != NULL && player2->new_state == 8) {
					if (gameType == StatusPanel::Client) {
						printf("green nuked\n");
						greenNuked = true;
					} else {
						printf("blue nuked\n");
						blueNuked = true;
					}
				}
			} else {
				gameData.nukeCount++;
			}

			gameData.thisTotalOut = gameData.totalOut = 0;

			Lemming* s;
			for (int n = 0; n < max_num_lemmings; n++) {
				s = ulemmings.lemmings[n];

				if (player2 != NULL) {
					DoLemmingAction(player2, s, n);
				}

				if (player1 != NULL) {
					if (DoLemmingAction(player1, s, n)) {
						if (!suppress_tool_decrease) {
							currentLevel->UseTool(
									(Level::ToolType) player1->new_state);
							samples->PlaySample(DOACTION);
						}
					}
				}

				int exit_reached = AnimateLemming(s);
				if (exit_reached != lemming_arrived_home) {
					//lemming's home, save the fact
					if (exit_reached == StatusPanel::Server) {
						//Lem reached a blue home but is he a blue/server?
						if (s->GetPlayerNum() == 1) {
							statusPanel->AddToBlueTotal(StatusPanel::Server);
						} else {
							statusPanel->AddToBlueTotal(StatusPanel::Client);
						}
					} else {
						//Lem reached a green home but is he a green/server?
						if (s->GetPlayerNum() == 1) {
							statusPanel->AddToGreenTotal(StatusPanel::Server);
						} else {
							statusPanel->AddToGreenTotal(StatusPanel::Client);
						}
					}
				}
			}

			if (!gameData.totalledUp && IsGameComplete()) {
				gameData.totalledUp = true;
				if (gameType != StatusPanel::SinglePlayer) {
					if (GreenHome() > BlueHome()) {
						greenGamesWon++;
					} else if (GreenHome() < BlueHome()) {
						blueGamesWon++;
					}

					greenGameMaxLems = min_multiplay_lemmings + GreenHome();
					if (greenGameMaxLems > max_num_lemmings / 2) {
						greenGameMaxLems = max_num_lemmings / 2;
					}

					blueGameMaxLems = min_multiplay_lemmings + BlueHome();
					if (blueGameMaxLems > max_num_lemmings / 2) {
						blueGameMaxLems = max_num_lemmings / 2;
					}
				}
			}
		}
	}
}

