#include "StatusPanel.h"
#include "Lemming.h"

int LEVEL_X_SIZE;
int LEVEL_Y_SIZE;

/*
 * IconDisplay.cpp
 *
 *  Created on: 16 Oct 2009
 *      Author: math
 */

StatusPanel::StatusPanel(int swidth, int sheight, int newXpos, int newYpos, SamplePlayer* newSamples, int Zoom)
{
	samples = newSamples;
	utility = GraphicsUtility::Instance();
	fontRenderer = FontRenderer::Instance();
	printf("Status Panel got: %d\n", swidth);
	if(swidth == 400){
		iconBar = utility->load_image("data/graphics/amigaIcons_800.png");
		iconBarClient = utility->load_image_swap("data/graphics/amigaIcons_800.png",
														SDL_MapRGB(iconBar->format, 85, 51, 254),
														SDL_MapRGB(iconBar->format, 0, 186, 0));
		offsetX = 441;
		iconWidth = 40;
		iconHeight = 55;
		selectedIconX = 80;
		OVERVIEW_OFFSET_X  = 517;
		OVERVIEW_OFFSET_Y  = 3;
		OVERVIEW_WIDTH  = 245;
		OVERVIEW_HEIGHT = 50;

	}else{
		iconBar = utility->load_image("data/graphics/amigaIcons.png");
		iconBarClient = utility->load_image_swap("data/graphics/amigaIcons.png",
																SDL_MapRGB(iconBar->format, 85, 51, 254),
																SDL_MapRGB(iconBar->format, 0, 186, 0));
		offsetX = 353;
		iconWidth = 32;
		iconHeight = 44;
		selectedIconX = 64;

		OVERVIEW_OFFSET_X  = 414;
		OVERVIEW_OFFSET_Y  = 2;
		OVERVIEW_WIDTH  = 196;
		OVERVIEW_HEIGHT = 40;
	}

	totalsImage = utility->load_image("data/graphics/TwoplayerTotals.png");
	zoom = Zoom;
	xpos = newXpos;
	ypos = newYpos;
	tool_complement = NULL;
	viewport.x = viewport.y = 0;
	viewport.w = swidth*zoom;
	viewport.h = sheight*zoom;
	iconSelected = CLIMBER;

	selectedCount = 0;
	levelOverview = NULL;
}

StatusPanel::~StatusPanel()
{
	SDL_FreeSurface(iconBar);
	SDL_FreeSurface(iconBarClient);
	SDL_FreeSurface(levelOverview);
	SDL_FreeSurface(totalsImage);
}

void StatusPanel::HideSelection(bool hide){
	hideSelection = hide;
}

void StatusPanel::SetGameMode(GameType new_type)
{
	type = new_type;
}

void StatusPanel::ClearTotals(){
	blueTotals.clear();
	greenTotals.clear();
}

int StatusPanel::BlueHome()
{
	return blueTotals.size();
}

int StatusPanel::GreenHome()
{
	return greenTotals.size();
}

void StatusPanel::AddToGreenTotal(GameType newOne)
{
	greenTotals.push_back(newOne);
}

void StatusPanel::AddToBlueTotal(GameType newOne)
{
	blueTotals.push_back(newOne);
}

void StatusPanel::CreateOverviewImage(SDL_Surface* fullSizeLevel)
{
	int x,y;
	overviewRect.w = overviewRect.h = 1;
	printf("In StatusPanel CreateOverViewImage\n");
	LEVEL_X_SIZE = fullSizeLevel->w;
	LEVEL_Y_SIZE = fullSizeLevel->h;

	divx = fullSizeLevel->w/OVERVIEW_WIDTH;
	divy = fullSizeLevel->h/OVERVIEW_HEIGHT;

	if(levelOverview == NULL)
	{
		levelOverview = SDL_CreateRGBSurface (fullSizeLevel->flags,
									OVERVIEW_WIDTH,
									OVERVIEW_HEIGHT,
									fullSizeLevel->format->BitsPerPixel,
									fullSizeLevel->format->Rmask,
									fullSizeLevel->format->Gmask,
									fullSizeLevel->format->Bmask,
									fullSizeLevel->format->Amask);
	}
	else
	{
		//Clear the screen
		SDL_FillRect( levelOverview, &levelOverview->clip_rect, SDL_MapRGB( levelOverview->format, 0x00, 0x00, 0x00 ) );
	}

	for (y = 1; y < fullSizeLevel->h; y++){
		overviewRect.y = y/divy;
		for (x = 1; x < fullSizeLevel->w; x++){
			overviewRect.x = x/divx;
			if(utility->getpixel(fullSizeLevel, x, y)){
				if(overviewRect.x >=0 && overviewRect.x < OVERVIEW_WIDTH && overviewRect.y < OVERVIEW_HEIGHT && overviewRect.y >= 0){
					SDL_FillRect( levelOverview, &overviewRect, SDL_MapRGB(levelOverview->format,0,100,0));
				}
			}
		}
	}

	markerColour = SDL_MapRGB(levelOverview->format, 175,175,0);
}

void StatusPanel::SetSelectedStatus(std::string type, int count)
{
	selectedType = type;
	selectedCount = count;
}

void StatusPanel::SetGameStats(int TimeRemaining, int TotalOut, int PercentHome, int Release_rate, std::vector<int>* counts, int MinReleaseRate)
{
	tool_complement = counts;
	minReleaseRate = MinReleaseRate;
	timeRemaining = TimeRemaining;
	totalOut = TotalOut;
	percentHome = PercentHome;
	release_rate = Release_rate;
	tool_complement = counts;
}

SDL_Rect r;
void StatusPanel::DrawMarker(int x,int y, SDL_Surface* screen, DrawMode drawmode)
{
	r.w=1;r.h=2;
	r.x = (xpos + x/divx + OVERVIEW_OFFSET_X );
	r.y = (ypos + y/divy + OVERVIEW_OFFSET_Y + FontHeight)+1;
	//and draw a little dot on the overview window:
	SDL_FillRect( screen, &r, markerColour);

	// now draw a dot or a black out a pixel
	r.w=1;r.h=1;
	r.x = (x/divx);
	r.y = (y/divy)+2;
	if(drawmode == Pixel){
		SDL_FillRect( levelOverview, &r, SDL_MapRGB(levelOverview->format,0,100,0));
	}else if(drawmode == NoPixel){
		SDL_FillRect( levelOverview, &r, SDL_MapRGB(iconBar->format, 0, 0, 0));
	}
}

void StatusPanel::Draw(SDL_Surface* screen)
{
	if(type == SinglePlayer || type == Server )
		Draw(screen, iconBar);
	else if(type == Client)
		Draw(screen, iconBarClient);
}

void StatusPanel::DrawTotals(SDL_Surface* screen){
	if(type != SinglePlayer){
		//Clear the screen
		SDL_FillRect(screen, &totalsImage->clip_rect, 0);
		utility->apply_surface(0,1, totalsImage, screen, &totalsImage->clip_rect);

		SDL_Rect clip;

		//now each thingy
		clip.y = 301;
		clip.x = 14;
		clip.w = 15;
		clip.h = 2;
		for(unsigned int n = 0; n < blueTotals.size(); n++)
		{
			if(blueTotals[n] == StatusPanel::Server)
				SDL_FillRect(screen, &clip, SDL_MapRGB(totalsImage->format, 85, 51, 254));
			else
				SDL_FillRect(screen, &clip, SDL_MapRGB(totalsImage->format, 0, 186, 0));

			clip.y -= 2;
		}

		clip.y = 301;
		clip.x = 30;
		clip.w = 15;
		clip.h = 2;
		//now each thingy
		for(unsigned int n = 0; n < greenTotals.size(); n++)
		{
			if(greenTotals[n] == StatusPanel::Server)
				SDL_FillRect(screen, &clip, SDL_MapRGB(totalsImage->format, 85, 51, 254));
			else
				SDL_FillRect(screen, &clip, SDL_MapRGB(totalsImage->format, 0, 186, 0));

			clip.y -= 2;
		}
	}
}

void StatusPanel::Draw(SDL_Surface* screen, SDL_Surface* icons)
{
	//Draw the icons
	SDL_Rect r;
	r.w = screen->w;
	r.h = screen->h - ypos;
	r.x = xpos;
	r.y = ypos;

	//Clear the screen
	SDL_FillRect( screen, &r, SDL_MapRGB( screen->format, 0x00, 0x00, 0x00 ) );

	utility->apply_surface( xpos, ypos+FontHeight, icons, screen, NULL );

	//Draw Overview
	if(levelOverview != NULL){
		utility->apply_surface( xpos + OVERVIEW_OFFSET_X, ypos + OVERVIEW_OFFSET_Y + FontHeight, levelOverview, screen, NULL );
	}

	//draw view port
	utility->DrawBox(screen, xpos + OVERVIEW_OFFSET_X + viewport.x/divx, ypos + FontHeight + OVERVIEW_OFFSET_Y + viewport.y/divy, viewport.w/divx, OVERVIEW_HEIGHT, SDL_MapRGB(screen->format, 0,187,0) );
	utility->DrawBox(screen, xpos + OVERVIEW_OFFSET_X + viewport.x/divx + 1, ypos + FontHeight + OVERVIEW_OFFSET_Y + viewport.y/divy - 1, viewport.w/divx, OVERVIEW_HEIGHT, SDL_MapRGB(screen->format, 0,187,0) );

	if(selectedCount > 0 )
	{
		std::stringstream selectedLemming;
		selectedLemming << selectedType;
		if(selectedCount > 1) selectedLemming  << " " << selectedCount;
		fontRenderer->RenderString(screen, selectedLemming.str(), xpos+(type == SinglePlayer ? 1 : 64), ypos, GREEN, ABSOLUTE);
	}

	int numPos = ypos + OVERVIEW_OFFSET_Y + FontHeight;

	std::stringstream StrStream;
	StrStream << totalOut;

	std::string outCount = "OUT ";
	outCount += StrStream.str();
	if(type == SinglePlayer)
		fontRenderer->RenderString(screen, outCount, xpos+200, ypos, GREEN, ABSOLUTE);
	else
		fontRenderer->RenderString(screen, outCount, xpos+250, ypos, GREEN, ABSOLUTE);

	//No time limit for multi-player
	//also, totals are graphical.
	if(type == SinglePlayer)
	{
		std::stringstream numHomeStr;
		numHomeStr << "";
		if(percentHome == 0)
		{
			numHomeStr << "00";
		}
		else
		{
			if(percentHome < 10) numHomeStr << "0";
			numHomeStr << percentHome;
		}

		std::string inCount = "IN ";
		inCount += numHomeStr.str();
		inCount += "%";
		fontRenderer->RenderString(screen, inCount, xpos+335, ypos, GREEN, ABSOLUTE);

		/*
		 * Display Time
		 */
		std::stringstream timeStream;
		timeStream.clear();
		timeStream << "TIME ";
		timeStream << timeRemaining/60;
		timeStream << "-";
		int timeRemDiv = (timeRemaining/60)*60;
		if(timeRemaining - timeRemDiv == 0){
			timeStream << "00";
		}
		else if (timeRemaining - timeRemDiv  < 10)
		{
			timeStream << "0";
			timeStream << timeRemaining - timeRemDiv;
		}
		else timeStream << timeRemaining - timeRemDiv;

		std::string timeLeft = timeStream.str();
		fontRenderer->RenderString(screen, timeLeft, screen->clip_rect.w-150, ypos, GREEN, ABSOLUTE);
	}

	int toolX;
	std::stringstream numberStream;
	if(tool_complement != NULL)
	{
		if(iconWidth == 40){
			toolX = iconWidth+iconWidth + 10;
		}
		else{
			toolX = iconWidth+iconWidth + 5;
		}

		for(int tool = 0; tool < 8; tool++)
		{
			if(tool_complement->at(tool) > 0)
			{
				numberStream.str("");
				if(tool_complement->at(tool) < 10){
					numberStream << "" << "0";
				}

				numberStream << "" << (int)(tool_complement->at(tool));

				fontRenderer->RenderNumber(screen, numberStream.str(), xpos+toolX, numPos);
			}
			toolX += iconWidth;
		}
	}

	if(iconWidth == 40){
		toolX = 10;
	}
	else{
		toolX = 5;
	}

	numberStream.str("");
	if(release_rate < 10){
		numberStream  << "0";
	}
	numberStream <<  (int)(release_rate);

	fontRenderer->RenderNumber(screen, numberStream.str(), toolX+iconWidth, numPos);

	numberStream.str("");
	if(minReleaseRate < 10){
			numberStream  << "0";
	}

	numberStream << (int)(minReleaseRate);
	fontRenderer->RenderNumber(screen, numberStream.str(), 8, numPos);

	//draw box over selected icon
	if(!hideSelection){
		utility->DrawBox(screen, xpos+selectedIconX,  ypos+FontHeight, iconWidth, iconHeight);
		utility->DrawBox(screen, xpos+selectedIconX+1,  ypos+FontHeight+1, iconWidth-2, iconHeight-1);
	}
}

void StatusPanel::SetSelectedIcon(int value)
{
	iconSelected = value;
	samples->PlaySample(CHANGEOP);
	int nsx = offsetX - iconWidth;

	if(iconSelected == DIGGER){
		selectedIconX = nsx - iconWidth;
	}else if(iconSelected == MINER){
		selectedIconX = nsx - (iconWidth*2);
	}else if(iconSelected == BASHER){
		selectedIconX = nsx - (iconWidth*3);
	}else if(iconSelected == BUILDER){
		selectedIconX = nsx - (iconWidth*4);
	}else if(iconSelected == BLOCKER){
		selectedIconX = nsx - (iconWidth*5);
	}else if(iconSelected == NUKE){
		selectedIconX = nsx - (iconWidth*6);
	}else if(iconSelected == FLOATER){
		selectedIconX = nsx - (iconWidth*7);
	}else if(iconSelected == CLIMBER){
		selectedIconX = nsx - (iconWidth*8);
	}
}

int StatusPanel::GetSelectedIcon()
{
	return iconSelected;
}


int StatusPanel::SelectIcon(int mouseX, int mouseY)
{
    int retVal = 0;
	mouseX -= xpos;

	if(mouseY > ypos+FontHeight)
	{
		retVal = true;
		if(mouseX > offsetX && mouseX <= offsetX+iconWidth){
			retVal = NUKE_BUTTON;
		}
		else
		{
			if(mouseX > offsetX-(iconWidth*1) && mouseX <= offsetX-(iconWidth*1)+iconWidth){
				retVal = PAUSE_BUTTON;
			}else if(mouseX > offsetX-(iconWidth*2) && mouseX <= offsetX-(iconWidth*2)+iconWidth){
				SetSelectedIcon(DIGGER);
			}else if(mouseX > offsetX-(iconWidth*3) && mouseX <= offsetX-(iconWidth*3)+iconWidth){
				SetSelectedIcon(MINER);
			}else if(mouseX > offsetX-(iconWidth*4) && mouseX <= offsetX-(iconWidth*4)+iconWidth){
				SetSelectedIcon(BASHER);
			}else if(mouseX > offsetX-(iconWidth*5) && mouseX <= offsetX-(iconWidth*5)+iconWidth){
				SetSelectedIcon(BUILDER);
			}else if(mouseX > offsetX-(iconWidth*6) && mouseX <= offsetX-(iconWidth*6)+iconWidth){
				SetSelectedIcon(BLOCKER);
			}else if(mouseX > offsetX-(iconWidth*7) && mouseX <= offsetX-(iconWidth*7)+iconWidth){
				SetSelectedIcon(NUKE);
			}else if(mouseX > offsetX-(iconWidth*8) && mouseX <= offsetX-(iconWidth*8)+iconWidth){
				SetSelectedIcon(FLOATER);
			}else if(mouseX > offsetX-(iconWidth*9) && mouseX <= offsetX-(iconWidth*9)+iconWidth){
				SetSelectedIcon(CLIMBER);
			}
		}
	}
	return retVal;
}

void StatusPanel::UpdateOverviewOffset(int mouseX, int mouseY)
{
    mouseX -= xpos;
	if(mouseY > ypos+FontHeight){
		if(mouseX >= OVERVIEW_OFFSET_X && mouseY >= OVERVIEW_OFFSET_Y){
			//now check the arrow is within the current
			viewport.x = ((mouseX - viewport.w/(LEVEL_X_SIZE/OVERVIEW_WIDTH)/2)-OVERVIEW_OFFSET_X )*(LEVEL_X_SIZE/OVERVIEW_WIDTH);
			if(viewport.x < 0){
				viewport.x = 0;
			}
			if(viewport.x > LEVEL_X_SIZE - (viewport.w)){
				viewport.x = LEVEL_X_SIZE - (viewport.w);
			}
		}
	}
}

void StatusPanel::MoveItemLeft(){
	if(GetSelectedIcon() == DIGGER){
		SetSelectedIcon(MINER);
	}else if(GetSelectedIcon() == MINER){
		SetSelectedIcon(BASHER);
	}else if(GetSelectedIcon() == BASHER){
		SetSelectedIcon(BUILDER);
	}else if(GetSelectedIcon() == BUILDER){
		SetSelectedIcon(BLOCKER);
	}else if(GetSelectedIcon() == BLOCKER){
		SetSelectedIcon(NUKE);
	}else if(GetSelectedIcon() == NUKE){
		SetSelectedIcon(FLOATER);
	}else if(GetSelectedIcon() == FLOATER){
		SetSelectedIcon(CLIMBER);
	}else if(GetSelectedIcon() == CLIMBER){
		SetSelectedIcon(DIGGER);
	}
}

void StatusPanel::MoveItemRight(){
	if(GetSelectedIcon() == DIGGER){
		SetSelectedIcon(CLIMBER);
	}else if(GetSelectedIcon() == CLIMBER){
		SetSelectedIcon(FLOATER);
	}else if(GetSelectedIcon() == FLOATER){
		SetSelectedIcon(NUKE);
	}else if(GetSelectedIcon() == NUKE){
		SetSelectedIcon(BLOCKER);
	}else if(GetSelectedIcon() == BLOCKER){
		SetSelectedIcon(BUILDER);
	}else if(GetSelectedIcon() == BUILDER){
		SetSelectedIcon(BASHER);
	}else if(GetSelectedIcon() == BASHER){
		SetSelectedIcon(MINER);
	}else if(GetSelectedIcon() == MINER){
		SetSelectedIcon(DIGGER);
	}
}

void StatusPanel::Scroll(int amount){
	viewport.x += amount;

	if(viewport.x < 0){
		viewport.x = 0;
	}

	if(viewport.x > LEVEL_X_SIZE - (viewport.w)){
		viewport.x = LEVEL_X_SIZE - (viewport.w);
	}
}
