/*
 * MenuItem.cpp
 *
 *  Created on: 10 Dec 2009
 *      Author: mat
 */

#include "MenuItem.h"

GraphicsUtility* MenuItem::utility = GraphicsUtility::Instance();
FontRenderer* MenuItem::fontRenderer = NULL;
LevelLoader* MenuItem::levelLoader = NULL;
SDL_Surface* MenuItem::hand = NULL;
SDL_Surface* MenuItem::sleepIcon = NULL;
SDL_Surface* MenuItem::background = NULL;
SDL_Surface* MenuItem::brown_tile = NULL;
SDL_Surface* MenuItem::tempScreen = NULL;
bool MenuItem::playingMusic = true;
Mix_Music* MenuItem::Mix = NULL;
int MenuItem::currentLevelSet = 0;
unsigned int MenuItem::currentSoundSetting = 0;
Game* MenuItem::game = NULL;
int MenuItem::screen_width = 320;
int MenuItem::screen_height = 200;
bool MenuItem::leftclick = false;
bool MenuItem::leftButtonDown = false;
MenuItem* MenuItem::home = NULL;
MenuItem* MenuItem::plevelSelect = NULL;
MenuItem* MenuItem::multiplayerLevelSelect = NULL;
int MenuItem::game_frame_mult = 2;
MenuItem* MenuItem::nextMenuItemObject = NULL;
Configuration* MenuItem::config = NULL;

SDL_Surface* MenuItem::temp = NULL;
int MenuItem::fadingOut = 0;
int MenuItem::fadingIn = 0;
bool MenuItem::forceRedraw = false;
int MenuItem::mouse_x = 0;
int MenuItem::mouse_y = 0;
std::string MenuItem::ipaddress;
bool MenuItem::sound_enabled = true;
bool MenuItem::sleeping = false;

#ifdef PLATFORM_PANDORA
bool MenuItem::showMousePointer = false;
#else
bool MenuItem::showMousePointer = true;
#endif

MenuItem::MenuItem(MenuItem* newParent) {
	if (home == NULL)
		home = this;
	parent = newParent;
	currentMenuItemObject = this;
	leftclick = false;
	drawhand = true;
	topfade = 172;
	fadingOut = topfade;
	fadingIn = topfade - 1;
	quitGame = false;

	//showbackbutton = false;

	greenTiledBackground = true;

	fadespeed = 25;
	showfocusrect = false;
}

MenuItem::MenuItem(MenuItem* newParent, int new_width, int new_height) {
	quitGame = false;
	if (home == NULL)
		home = this;
	currentMenuItemObject = this;
	parent = newParent;
	drawhand = true;
	leftclick = false;
	topfade = 172;
	fadingOut = topfade;
	fadingIn = topfade - 1;
	screen_width = new_width;
	screen_height = new_height;
	leftButtonDown = false;

	greenTiledBackground = true;
	fadespeed = 25;
	showfocusrect = false;

	if (config == NULL) {
		config = new Configuration();
		config->Load();
	}

	if (Mix == NULL)
		Mix = Mix_LoadMUS("data/music/track13Remix.mp3");

	if (background == NULL){
		background = utility->load_image_zoomed(
				"data/graphics/lemmings_background_fs.png", 2);
	}

	if (brown_tile == NULL)
		brown_tile = utility->load_image_zoomed(
					"data/graphics/tile_brown.png", 2);

	if (hand == NULL) {
		hand = utility->load_image("data/graphics/hand.png");
		SDL_SetColorKey(hand, SDL_SRCCOLORKEY, SDL_MapRGB(hand->format, 255, 0,
				255));
	}

	if (sleepIcon == NULL) {
		sleepIcon = utility->load_image("data/graphics/sleep.png");
		SDL_SetColorKey(sleepIcon, SDL_SRCCOLORKEY, SDL_MapRGB(
				sleepIcon->format, 255, 0, 255));
	}

	if (levelLoader == NULL) {
		levelLoader = new LevelLoader();
		levelLoader->Setup();
	}

	if (fontRenderer == NULL) {
		fontRenderer = FontRenderer::Instance();
	}
}

MenuItem::~MenuItem() {
	if (!parent) {
		SDL_FreeSurface(background);
		SDL_FreeSurface(hand);
		delete (config);
	}
}

bool MenuItem::HasUserQuit() {
	return quitGame;
}

void MenuItem::TileBackground(SDL_Surface* screen) {
	if(	greenTiledBackground ){
		for(int x = 0; x < screen->w; x+= background->w )
			for(int y = 0; y < screen->h; y+= background->h )
		utility->apply_surface(x, y, background, screen,
				&background->clip_rect);
	}
	else{
		for(int x = 0; x < screen->w; x+= background->w )
			for(int y = 0; y < screen->h; y+= background->h )
				utility->apply_surface(x, y, brown_tile, screen,
					&brown_tile->clip_rect);
	}
}

/*
 * Draws the screen calls virtual draw then
 */
void MenuItem::Draw(SDL_Surface* screen) {

	/*THIS NEEDS MOVING*/
	if (temp == NULL) {
		temp = SDL_CreateRGBSurface(screen->flags, screen->w, screen->h,
				screen->format->BitsPerPixel, screen->format->Rmask,
				screen->format->Gmask, screen->format->Bmask,
				screen->format->Amask);
	}

	if (tempScreen == NULL) {
		tempScreen = SDL_CreateRGBSurface(screen->flags, screen->w, screen->h,
				screen->format->BitsPerPixel, screen->format->Rmask,
				screen->format->Gmask, screen->format->Bmask,
				screen->format->Amask);
	}

	DrawSelf(tempScreen);

	//fading in or out or not?
	if (fadingOut < topfade) {
		forceRedraw = true;
		SDL_SetAlpha(temp, SDL_SRCALPHA, fadingOut);
		SDL_BlitSurface(temp, NULL, tempScreen, &tempScreen->clip_rect);
		fadingOut += fadespeed;
		if (fadingOut >= topfade - 1) {
			fadingOut = topfade;
			fadingIn = topfade - 1;
			if (nextMenuItemObject != NULL) {
				currentMenuItemObject = nextMenuItemObject;
				nextMenuItemObject = NULL;
				forceRedraw = true;
			}
		}
	} else if (fadingIn > 0) {
		forceRedraw = true;
		//fade out current screen
		SDL_SetAlpha(temp, SDL_SRCALPHA, fadingIn);
		SDL_BlitSurface(temp, NULL, tempScreen, &tempScreen->clip_rect);

		fadingIn -= fadespeed;
		if (fadingIn <= fadespeed) {
			fadingIn = -fadespeed;
			fadingOut = topfade;
			forceRedraw = true;
		}
	}

	utility->apply_surface(0, 0, tempScreen, screen, NULL);

	if (!sleeping) {
		for (unsigned int n = 0; n < clickAreas.size(); n++) {
			RenderClickArea(screen, &clickAreas.at(n));
		}
	}

	if (showMousePointer){
		DrawMousePointer(screen);
	}
}

void MenuItem::AddClickArea(int x,int y,int w,int h, std::string t, MenuItem* newScreen, SDL_Surface* image){
	AddClickArea(x,y,w,h,t,NULL,newScreen,image);
}

void MenuItem::AddClickArea(int x, int y, int w, int h, std::string text,
		MenuItem::callback p) {
	AddClickArea(x,y,w,h,text,p,NULL, NULL);
}

void MenuItem::AddClickArea(int x, int y, int w, int h, std::string text,
		MenuItem* newScreen) {
	AddClickArea(x,y,w,h,text,NULL,newScreen,NULL);
}

void MenuItem::AddClickArea(int x, int y, int w, int h, std::string text, MenuItem::callback p,
		MenuItem* newScreen, SDL_Surface* image) {
	ClickArea a;
	a.rect.x = x;
	a.rect.y = y;
	a.rect.w = w;
	a.rect.h = h;
	a.over = false;
	a.func = p;
	a.newScreen = newScreen;
	a.msg = text;
	a.image = image;
	clickAreas.push_back(a);
}

void MenuItem::RenderClickArea(SDL_Surface *screen, ClickArea* c) {
	if(c->msg.length() > 0){
		std::stringstream stream;
		stream.clear();
		stream.str("");
		stream << c->msg;
		fontRenderer->RenderString(screen, stream.str(), c->rect.x + 5, c->rect.y,
				GREEN, ABSOLUTE);
	}

	if(c->image != NULL){
		utility->apply_surface(c->rect.x, c->rect.y, c->image, screen, &c->image->clip_rect );
	}

	if (c->over && showfocusrect && showMousePointer) {
		utility->DrawBox(screen, &c->rect, c->rect.x, c->rect.y + 5);
	}
}

bool MenuItem::CheckClickArea(int x, int y, ClickArea* c) {
	c->over = (x > c->rect.x && x < (c->rect.x + c->rect.w) && y > c->rect.y
			&& y < (c->rect.y + c->rect.h));

	if (leftclick && c->over) {
		if (c->func != NULL) {
			((this->*c->func)());
		} else if (c->newScreen != NULL) {
			SetCurrentScreen(c->newScreen);
		}
	}

	return c->over;
}

void MenuItem::DrawMousePointer(SDL_Surface* screen) {
	if (drawhand)//TODO: this is lame
	{
		if (sleeping)
			utility->apply_surface(mouse_x, mouse_y, sleepIcon, screen, NULL);
		else
			utility->apply_surface(mouse_x, mouse_y, hand, screen, NULL);
	}
}

MenuItem* MenuItem::SetCurrentScreen(MenuItem* item) {
	if (scrlock()) {
		Mix_FadeOutMusic(1000);
		playingMusic = false;
		if (item != NULL) {
			nextMenuItemObject = item->Reset();
			fadingOut = 0;
			fadingIn = 0;
		}
	}
	return this;
}

void MenuItem::GoBack() {
	SendEscape();
}

void MenuItem::GoForward() {
	OnGoForward();
}

void MenuItem::OnGoForward() {
}

MenuItem* MenuItem::SendEscape() {
	if (scrlock()) {
		if (parent != NULL) {
			return SetCurrentScreen(parent);
		} else {
			//This is the end, beautiful friend:
			config->Save();
			quitGame = true;
		}
	} else
		return this;
}

/*
 * Returns true if the menu system is fading in or out
 */
bool MenuItem::scrlock() {
	return !(fadingOut < topfade || fadingIn > 0);
}

MenuItem* MenuItem::SetMouse(int x, int y, int leftdown, int rightdown) {
	leftclick = false;

	mouse_x = x;
	mouse_y = y;

	if (leftdown && scrlock()) {
		//but was it before?
		if (!leftButtonDown) {
			//no, it wasn't, we can send it
			leftclick = leftButtonDown = true;
		}
	} else {
		//button not, down
		leftButtonDown = false;
	}

	for (unsigned int n = 0; n < clickAreas.size(); n++) {
		CheckClickArea(x, y, &clickAreas.at(n));
	}

	HandleMouse(x, y, leftdown, rightdown);

	return currentMenuItemObject;
}

bool MenuItem::SetKey(int k) {
	if (scrlock()) {
		HandleKey(k);
	}
	return true;
}

void MenuItem::Init() {
	forceRedraw = true;
	sleeping = false;
}

void MenuItem::DrawLevelPreview(SDL_Surface* screen) {
	SDL_Surface* preview = game->GetLevelPreview();
	if (preview != NULL) {
		SDL_Rect blu;
		blu.h = preview->clip_rect.h + 40;
		blu.w = screen->clip_rect.w;
		blu.x = 0;
		blu.y = 0;
		SDL_FillRect(screen, &blu, SDL_MapRGB(screen->format, 0x00, 0x00, 0x33));
		utility->apply_surface(
				(screen->clip_rect.w - preview->clip_rect.w) / 2, 25, preview,
				screen, NULL);
	}
}

MenuItem* MenuItem::Reset() {
	currentMenuItemObject = this;
	Init();
	return this;
}

void MenuItem::FreeMusic() {
	//this caused so many problems!
	//Mix_FreeMusic(Mix);
}

void MenuItem::PreviousLevel() {
	if (levelLoader->currentSinglePlayerLevel > 0) {
		levelLoader->PreviousLevel();
		Init();
	}
}

void MenuItem::NextLevel() {
	levelLoader->NextLevel();
	Init();
}

void MenuItem::KeyUp(int k) {

}

void MenuItem::KeyDown(int k) {
#ifdef PLATFORM_PANDORA
	if(k == SDLK_c){
		showMousePointer = !showMousePointer;
		game->showLemmingInfo = !game->showLemmingInfo;
	}
#endif
}
