/*
 * MenuItem.h
 *
 *  Created on: 10 Dec 2009
 *      Author: mat
 */

#ifndef MENUITEM_H_
#define MENUITEM_H_

#include "SDL/SDL.h"
#include "SDL/SDL_mixer.h"
#include "../GraphicsUtility.h"
#include "../FontRenderer.h"
#include "../LevelLoader.h"
#include <vector>
#include <sstream>
#include "../Game.h"
#include "../Configuration.h"

using namespace std;

#define DRAW_SELF -1

class MenuItem {
public:
	typedef void (MenuItem::*callback)( void );
	MenuItem(MenuItem*);
	MenuItem(MenuItem*, int, int);
	virtual ~MenuItem();

	void Draw(SDL_Surface*);
	MenuItem* SetMouse(int, int, int, int);
	bool SetKey(int);
	virtual void KeyUp(int);
	virtual void KeyDown(int);
	virtual void Init();
	virtual MenuItem* SendEscape();
	MenuItem* Reset();
	bool HasUserQuit();
	static LevelLoader* levelLoader;
	static bool showMousePointer;
	static Configuration* config;
	void GoForward();
	void GoBack();
	virtual void PreviousLevel();
	virtual void NextLevel();
protected:
	virtual void DrawSelf(SDL_Surface*) = 0;
	virtual void HandleKey(int) = 0;
	virtual void HandleMouse(int, int, long, long) = 0;
	virtual void DrawMousePointer(SDL_Surface*);
	virtual void OnGoForward();

	static void FreeMusic();
	int AddMenuItem(MenuItem* item);
	void TileBackground(SDL_Surface*);
	MenuItem* SetCurrentScreen(MenuItem* item);
	void DrawLevelPreview(SDL_Surface*);
	bool scrlock();
	static int mouse_x;
	static int mouse_y;
	static bool leftclick;
	static bool sleeping;
	bool drawhand;
	MenuItem* parent;
	bool quitGame;
	static bool sound_enabled;
	static GraphicsUtility* utility;
	static FontRenderer* fontRenderer;
	static bool playingMusic;
	static Mix_Music *Mix;
	static int currentLevelSet;
	static unsigned int currentSoundSetting;
	static int screen_width;
	static int screen_height;
	static Game* game;
	static SDL_Surface* background;
	static SDL_Surface* brown_tile;
	static SDL_Surface* temp;
	static SDL_Surface* tempScreen;
	static bool forceRedraw;
	static SDL_Surface* hand;
	static SDL_Surface* sleepIcon;
	static MenuItem* home;
	static MenuItem* plevelSelect;
	static MenuItem* multiplayerLevelSelect;
	static int game_frame_mult;
	static MenuItem* nextMenuItemObject;
	static std::string ipaddress;

	//bool showbackbutton;
	int fadespeed;
	int topfade;
	struct ClickArea{
		SDL_Rect rect;
		bool over;
		std::string msg;
		MenuItem::callback func;
		MenuItem* newScreen;
		SDL_Surface* image;
	};

	ClickArea nextClickArea;
	std::vector<ClickArea> clickAreas;

	void AddClickArea(int,int,int,int, std::string, MenuItem::callback);
	void AddClickArea(int,int,int,int, std::string, MenuItem* newScreen);
	void AddClickArea(int,int,int,int, std::string, MenuItem* newScreen, SDL_Surface* image);
	void AddClickArea(int x, int y, int w, int h, std::string text, MenuItem::callback p, MenuItem* newScreen, SDL_Surface* image);
	void RenderClickArea(SDL_Surface *screen, ClickArea* clickArea);
	bool CheckClickArea(int x, int y, ClickArea* c);
	bool greenTiledBackground;

	bool showfocusrect;
private:
	static bool leftButtonDown;
	static int fadingOut;
	static int fadingIn;
	MenuItem* currentMenuItemObject;
};

#endif /* MENUITEM_H_ */
