/*
 * OnePlayerLevelSelect.cpp
 *
 *  Created on: 10 Dec 2009
 *      Author: math
 */

#include "OnePlayerLevelSelect.h"
#include "OnePlayerGameScreen.h"
#include <time.h>

int stampx;
int stampy;

OnePlayerLevelSelect::OnePlayerLevelSelect(MenuItem* newParent) : MenuItem(newParent) {
	if(plevelSelect == NULL) plevelSelect = this;
	gameScreenObject = new OnePlayerGameScreen(newParent);

	if(screen_width == 800){
		xoffset = 155;
	}
	else{
		xoffset = 120;
	}
	AddClickArea(10,screen_height - 40,40,40, "<<", &MenuItem::GoBack );
	AddClickArea(screen_width - 60,30,50,50,">", &MenuItem::NextLevel);
	AddClickArea(20,30,50,50,"<", &MenuItem::PreviousLevel);
	AddClickArea(xoffset,130,500,270,"", &MenuItem::GoForward);

	completed = GraphicsUtility::Instance()->load_image("data/graphics/completed.png");
}

OnePlayerLevelSelect::~OnePlayerLevelSelect()  {
	delete(gameScreenObject);
}

void OnePlayerLevelSelect::SetLevelInformation(string new_title, int new_numberOfLemmings, int new_precentToBeSaved, int new_releaseRate, int new_time, string new_rating)
{
	levelNumber = levelLoader->currentSinglePlayerLevel+1;
	title = new_title;
	numberOfLemmings = new_numberOfLemmings;
	precentToBeSaved = new_precentToBeSaved;
	releaseRate = new_releaseRate;
	time = new_time;
}

void OnePlayerLevelSelect::DrawSelf(SDL_Surface* screen){
	std::stringstream stream;
	stream.clear();

	//clear screen
	if(forceRedraw){
		SDL_FillRect(screen, &screen->clip_rect, SDL_MapRGB( screen->format, 0x00, 0x00, 0x00 ) );
		TileBackground(screen);
		DrawLevelPreview(screen);

		int xPos = xoffset+30;
		int yPos = 180;
		int h = 32;
		stream << "Level " << levelLoader->currentSinglePlayerLevel+1;
		fontRenderer->RenderString(screen, stream.str(), 1, yPos-32, RED, ABSOLUTE);

		stream.str("");
		stream << game->currentLevel->GetLevelName();
		fontRenderer->RenderString(screen, stream.str(), xPos, yPos-32, RED, ABSOLUTE);

		stream.str("");
		stream << "Number of Lemmings " << numberOfLemmings;
		fontRenderer->RenderString(screen, stream.str(), xPos, yPos, BLUE, ABSOLUTE);

		stream.str("");
		stream << precentToBeSaved << "% To Be Saved";
		fontRenderer->RenderString(screen, stream.str(), xPos, yPos+=h, GREEN, ABSOLUTE);

		stream.str("");
		stream << "Release Rate " << releaseRate;
		fontRenderer->RenderString(screen, stream.str(), xPos, yPos+=h, BROWN, ABSOLUTE);

		stream.str("");
		stream << "Time  " << time << " Minutes";
		fontRenderer->RenderString(screen, stream.str(), xPos, yPos+=h, LIGHT_BLUE, ABSOLUTE);

		stream.str("");
		stream << "Rating  " << levelLoader->GetLevelSetName();
		fontRenderer->RenderString(screen, stream.str(), xPos, yPos+=h, PURPLE, ABSOLUTE);

		if(config->GetLevelComplete(levelLoader->CurrentLevelCode())){
			utility->apply_surface(stampx, stampy, completed, screen, NULL);
		}
#ifdef PLATFORM_PANDORA
		fontRenderer->RenderString(screen, "Tap here to continue", 40, 390-FontHeight-2, BLUE, CENTER);
#else
		fontRenderer->RenderString(screen, "Click here to continue", 40, 390-FontHeight-2, BLUE, CENTER);
#endif
		forceRedraw = false;
	}
}

void OnePlayerLevelSelect::HandleKey(int k){
	if(k == SDLK_RETURN || k == SDLK_SPACE)
	{
		SetCurrentScreen(gameScreenObject);
	}
	else if(k == SDLK_RIGHT)
	{
		NextLevel();
	}
	else if(k == SDLK_LEFT)
	{
		PreviousLevel();
	}
	else if(k == SDLK_UP)
	{
		//if(levelLoader < 256){
			levelLoader->IncLevel(10);
			Init();
		//}
	}
	else if(k == SDLK_DOWN)
	{
		if(levelLoader->currentSinglePlayerLevel > 0){
			levelLoader->IncLevel(-10);
			Init();
		}
	}
}

void OnePlayerLevelSelect::OnGoForward(){
	SetCurrentScreen(gameScreenObject);
}

void OnePlayerLevelSelect::HandleMouse(int mx,int my,long lb,long rb){
}

void OnePlayerLevelSelect::Init(){
	if(game != NULL){
		game->LoadLevel(levelLoader);
		if( levelLoader->SuperLemming() ){
			game->super_lem_hack = true;
			gameScreenObject->SuperSpeed();
		}
		else{
			game->super_lem_hack = false;
			gameScreenObject->SetNormalSpeed();
		}
		game->Init(StatusPanel::SinglePlayer);

		SetLevelInformation(game->currentLevel->GetLevelName(),
							game->currentLevel->GetNumLemmings(),
							game->LemmingsNeededPercent(),
							game->currentLevel->GetReleaseRate(),
							game->currentLevel->GetTimeInMinutes(),
							game->currentLevel->GetRatingDescription());

		int amount = 10;
		if(rand() % 10 > 5){
			stampx = screen_width/2 - completed->clip_rect.w/2 + (rand() % amount);
		}
		else{
			stampx = screen_width/2 - completed->clip_rect.w/2 - (rand() % amount);
		}

		if(rand() % 10 > 5){
			stampy = 40 + (rand() % amount);
		}else{
			stampy = 40 - (rand() % amount);
		}
	}
	MenuItem::Init();
}


